//========================================================================================
//
//	File:				Actions.h
//	Release Version:	$ ODF 2 $
//
//	Author:				Jim Lloyd
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef ACTIONS_H
#include "Actions.h"
#endif

#ifndef DEFINES_K
#include "Defines.k"
#endif

#ifndef BINDING_K
#include "Binding.k"
#endif

#ifndef ENVIRONS_H
#include "Environs.h"
#endif

#ifndef FWKIND_H
#include "FWKind.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWSUUTIL_H
#include "FWSUUtil.h"
#endif

#ifndef FWALERT_H
#include "FWAlert.h"
#endif

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWBARRAY_H
#include "FWBArray.h"
#endif

#ifndef FWMEMMGR_H
#include "FWMemMgr.h"
#endif

#ifndef FWMEMHLP_H
#include "FWMemHlp.h"
#endif

#ifndef FWMEMHLP_H
#include "FWMemHlp.h"
#endif

#ifndef FWFILESP_H
#include "FWFileSp.h"
#endif

#ifndef FWRESOUR_H
#include "FWResour.h"
#endif

#ifndef FWACQUIR_H
#include "FWAcquir.h"
#endif

#ifndef FWSUSINK_H
#include "FWSUSink.h"
#endif

#ifndef FWSTRING_H
#include "FWString.h"
#endif

#ifndef FWODTYPS_H
#include "FWODTyps.h"
#endif

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_ODStorageUnit_xh
#include <StorageU.xh>
#endif

#ifndef SOM_ODTranslation_xh
#include <Translt.xh>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef SOM_ODDragItemIterator_xh
#include <DgItmIt.xh>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__DRAG__)
#include <Drag.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__SOUND__)
#include <Sound.h>
#endif

//========================================================================================
//	Static Allocations
//========================================================================================

ComponentInstance CScriptAction::fgComponentInstance;

//========================================================================================
//	Runtime information
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment odfbutton
#endif

//========================================================================================
//	class CAction
//========================================================================================

FW_DEFINE_CLASS_M0(CAction)

//----------------------------------------------------------------------------------------
//	CAction::CAction
//----------------------------------------------------------------------------------------

CAction::CAction(short actionType) :
	fActionType(actionType)
{
}

//----------------------------------------------------------------------------------------
//	CAction::~CAction
//----------------------------------------------------------------------------------------

CAction::~CAction()
{
}

//========================================================================================
//	class CSoundAction
//========================================================================================

FW_DEFINE_CLASS_M1(CSoundAction, CAction)

const FW_ClassTypeConstant LSoundAction = FW_TYPE_CONSTANT('S','N','D','A');
FW_REGISTER_ARCHIVABLE_CLASS(LSoundAction, CSoundAction, CSoundAction::Read, 0, 0, CSoundAction::Write)

//----------------------------------------------------------------------------------------
//	CSoundAction::CSoundAction
//----------------------------------------------------------------------------------------

CSoundAction::CSoundAction(Environment* ev, ODStorageUnit* storage, FW_CKind* kind):
	CAction(kSound),
	fSoundHandle(0)
{
	Internalize(ev, storage, kind);
}

//----------------------------------------------------------------------------------------
//	CSoundAction::CSoundAction
//----------------------------------------------------------------------------------------

CSoundAction::CSoundAction(Environment* ev, FW_CReadableStream& stream) :
	CAction(kSound),
	fSoundHandle(0)
{
	InternalizeFromStream(ev, stream);
}

//----------------------------------------------------------------------------------------
//	CSoundAction::~CSoundAction
//----------------------------------------------------------------------------------------

CSoundAction::~CSoundAction()
{
	if (fSoundHandle)
		::DisposeHandle(fSoundHandle);
}

//----------------------------------------------------------------------------------------
//	CSoundAction::Internalize
//----------------------------------------------------------------------------------------

void CSoundAction::Internalize(Environment* ev, ODStorageUnit* storage, FW_CKind* kind)
{
	if (kind->IsEqual(ev, 'snd ', kODPlatformDataType))
	{
		// Mac 'snd ' in Scrap
		FW_PStorageUnitSink suSink(ev, storage, kODPropContents, kind->GetType(ev));
		FW_CReadableStream stream(suSink);
		InternalizeFromStream(ev, stream);
	}
	else if (kind->IsEqual(ev, 'sfil', kODPlatformFileType))	
	{
		InternalizeSoundFile(ev, storage);
	}
}

//----------------------------------------------------------------------------------------
//	CSoundAction::InternalizeFromStream
//----------------------------------------------------------------------------------------

void CSoundAction::InternalizeFromStream(Environment* ev, FW_CReadableStream& stream)
{
	unsigned long size = stream.GetSink()->GetReadableBytes(ev);
	
	if (size > 0)
	{
		FW_CAcquireTemporarySystemHandle handle(size);
		stream.Read(handle.GetPointer(), size);
		fSoundHandle = handle.GetPlatformHandle();
		handle.Orphan();
	}
}

//----------------------------------------------------------------------------------------
//	CSoundAction::InternalizeSoundFile
//----------------------------------------------------------------------------------------

void CSoundAction::InternalizeSoundFile(Environment* ev, ODStorageUnit* storage)
{
	HFSFlavor hfsInfo;
	if (FW_MacSUReadHFSFlavor(ev, storage, kODPropContents, hfsInfo))
	{
		// Create platform independent file spec
		FW_PFileSpecification fileSpec(ev, hfsInfo.fileSpec);
		
		// Open the resource file
		FW_PResourceFile resourceFile(ev, fileSpec);

		// Get any (i.e the first) sound resource
		const FW_ResourceType resourceType = 'snd ';
		const FW_ResourceID resourceID = 1;
		Handle sound = ::Get1IndResource(resourceType, resourceID);
					
		if (sound != NULL)
		{
			// if succeeded, detach the handle and keep it
			::DetachResource(sound);
			fSoundHandle = sound;
		}
		else
		{
			// No sound resource loaded, throw an exception
			FW_FailOnError(::ResError());
			FW_Failure(resNotFound);
		}
	}
}

//----------------------------------------------------------------------------------------
//	CSoundAction::Externalize
//----------------------------------------------------------------------------------------

void CSoundAction::Externalize(Environment* ev, ODStorageUnit* storage, FW_CKind* kind)
{
	if (fSoundHandle != NULL)
	{
		FW_PStorageUnitSink suSink(ev, storage, kODPropContents, kind->GetType(ev));
		FW_CWritableStream stream(suSink);
		ExternalizeToStream(ev, stream);
		FW_SUDeleteEndOfFocusedValue(ev, storage);
	}
	else
	{
		FW_SURemoveValue(ev, storage, kODPropContents, kind->GetType(ev));
	}
}

//----------------------------------------------------------------------------------------
//	CSoundAction::ExternalizeToStream
//----------------------------------------------------------------------------------------

void CSoundAction::ExternalizeToStream(Environment* ev, FW_CWritableStream& stream)
{
FW_UNUSED(ev);
	if (fSoundHandle != NULL)
	{
		FW_CAcquireLockedSystemHandle lock(fSoundHandle);
		stream.Write(*fSoundHandle, FW_CMemoryManager::GetSystemHandleSize(fSoundHandle));
	}
}

//----------------------------------------------------------------------------------------
// CSoundAction::DoIt
//----------------------------------------------------------------------------------------

void CSoundAction::DoIt()
{
	if (fSoundHandle != NULL)
	{
		FW_CAcquireLockedSystemHandle lock(fSoundHandle);
		::SndPlay(NULL, (SndListHandle)fSoundHandle, TRUE);
	}
}

//----------------------------------------------------------------------------------------
// CSoundAction::Read
//----------------------------------------------------------------------------------------

void* CSoundAction::Read(FW_CReadableStream& stream, FW_ClassTypeConstant type)
{
FW_UNUSED(type);
	FW_SOMEnvironment ev;
	return new CSoundAction(ev, stream);
}

//----------------------------------------------------------------------------------------
// CSoundAction::Write
//----------------------------------------------------------------------------------------

void CSoundAction::Write(FW_CWritableStream& stream, FW_ClassTypeConstant type, const void *object)
{
FW_UNUSED(type);
	FW_SOMEnvironment ev;
	CSoundAction* action = (CSoundAction*) object;
	action->ExternalizeToStream(ev, stream);
}

//========================================================================================
//	class CScriptAction
//========================================================================================

FW_DEFINE_CLASS_M1(CScriptAction, CAction)

const FW_ClassTypeConstant LScriptAction = FW_TYPE_CONSTANT('S','C','R','A');
FW_REGISTER_ARCHIVABLE_CLASS(LScriptAction, CScriptAction, CScriptAction::Read, 0, 0, CScriptAction::Write)

//----------------------------------------------------------------------------------------
//	CScriptAction::CScriptAction
//----------------------------------------------------------------------------------------

CScriptAction::CScriptAction(Environment* ev, ODStorageUnit* storage, FW_CKind* kind) :
	CAction(kScript),
	fScriptHandle(0)
{
	Internalize(ev, storage, kind);
}

//----------------------------------------------------------------------------------------
//	CScriptAction::CScriptAction
//----------------------------------------------------------------------------------------

CScriptAction::CScriptAction(Environment* ev, FW_CReadableStream& stream) :
	CAction(kScript),
	fScriptHandle(0)
{
	InternalizeFromStream(ev, stream);
}

//----------------------------------------------------------------------------------------
//	CScriptAction::~CScriptAction
//----------------------------------------------------------------------------------------

CScriptAction::~CScriptAction()
{
	if (fScriptHandle)
		::DisposeHandle(fScriptHandle);
	ReleaseScriptingComponent();
}

//----------------------------------------------------------------------------------------
//	CScriptAction::Internalize
//----------------------------------------------------------------------------------------

void CScriptAction::Internalize(Environment* ev, ODStorageUnit* storage, FW_CKind* kind)
{
	if (kind->IsEqual(ev, 'scpt', kODPlatformDataType))
	{
		// 'scpt' on Scrap
		FW_PStorageUnitSink suSink(ev, storage, kODPropContents, kind->GetType(ev));
		FW_CReadableStream stream(suSink);
		InternalizeFromStream(ev, stream);
	}
	else if (kind->IsEqual(ev, 'osas', kODPlatformFileType))	// compiled script file
	{
		InternalizeScriptFile(ev, storage);
	}
}

//----------------------------------------------------------------------------------------
//	CScriptAction::InternalizeFromStream
//----------------------------------------------------------------------------------------

void CScriptAction::InternalizeFromStream(Environment* ev, FW_CReadableStream& stream)
{
	unsigned long size = stream.GetSink()->GetReadableBytes(ev);
	if (size > 0)
	{
		FW_CAcquireTemporarySystemHandle handle(size);
		stream.Read(handle.GetPointer(), size);
		fScriptHandle = handle.GetPlatformHandle();
		handle.Orphan();
	}
}

//----------------------------------------------------------------------------------------
//	CScriptAction::InternalizeScriptFile
//----------------------------------------------------------------------------------------

void CScriptAction::InternalizeScriptFile(Environment* ev, ODStorageUnit* storage)
{
	HFSFlavor hfsInfo;
	if (FW_MacSUReadHFSFlavor(ev, storage, kODPropContents, hfsInfo))
	{		
		// Make the file specification
		FW_PFileSpecification fileSpec(ev, hfsInfo.fileSpec);
		
		// Acquire an opened resource file, 
		// will be closed automatically
		FW_PResourceFile resourceFile(ev, fileSpec);
		
		// Acquire an opened resource, 
		// will be released automatically
		FW_PResource resource(ev, resourceFile, 128, 'scpt');
		FW_CAcquireResourceData lockedResource(ev, resource);
		
		// Copy the resource data into a new handle
		FW_CAcquireTemporarySystemHandle lockedHandle(lockedResource.GetSize());
		FW_CMemoryManager::CopyMemory(lockedResource.GetData(), lockedHandle.GetPointer(), lockedResource.GetSize());
		fScriptHandle = lockedHandle.GetPlatformHandle();
		lockedHandle.Orphan();
	}
}

//----------------------------------------------------------------------------------------
//	CScriptAction::Externalize
//----------------------------------------------------------------------------------------

void CScriptAction::Externalize(Environment* ev, ODStorageUnit* storage, FW_CKind* kind)
{
	if (fScriptHandle != NULL)
	{	
		FW_PStorageUnitSink suSink(ev, storage, kODPropContents, kind->GetType(ev));
		FW_CWritableStream stream(suSink);
		ExternalizeToStream(ev, stream);
		FW_SUDeleteEndOfFocusedValue(ev, storage);
	}
	else
	{
		FW_SURemoveValue(ev, storage, kODPropContents, kind->GetType(ev));
	}
}

//----------------------------------------------------------------------------------------
//	CScriptAction::ExternalizeToStream
//----------------------------------------------------------------------------------------

void CScriptAction::ExternalizeToStream(Environment* ev, FW_CWritableStream& stream)
{
FW_UNUSED(ev);
	if (fScriptHandle != NULL)
	{
		FW_CAcquireLockedSystemHandle lock(fScriptHandle);
		stream.Write(*fScriptHandle, FW_CMemoryManager::GetSystemHandleSize(fScriptHandle));
	}
}

//----------------------------------------------------------------------------------------
//	LoadAndExecuteScript
//----------------------------------------------------------------------------------------
// This function was lifted almost verbatim from
// IM:IAC, page 10-16. (Actually, I used MPTA, but it says 
// the code is on the above page)

void CScriptAction::LoadAndExecuteScript(Handle scriptData)
{
	AEDesc scriptDesc;
	OSAID scriptID;
	OSAID resultID;
	OSAError error;
	
	if (!HasAppleScript())
	{
		FW_CString32 name("ODFButton");
		FW_CString message;
		FW_SOMEnvironment ev;
		FW_PSharedLibraryResourceFile resFile(ev);
		
		::FW_LoadStringByID(ev, resFile, kButtonDialogStrings, FW_kMultiStringRes, kNoAppleScriptMsg, message);
		::FW_NoteAlert(name, message);
	}
	else
	{	
		// load the script data
		scriptDesc.descriptorType = typeOSAGenericStorage;
		scriptDesc.dataHandle = scriptData;
		
		ComponentInstance componentInstance = GetScriptingComponent();
		
		error = ::OSALoad(componentInstance, &scriptDesc, kOSAModeNull, &scriptID);
		
		if (error == noErr)
		{ 
			// execute the compiled script in the default context				 
			error = ::OSAExecute(componentInstance, scriptID, kOSANullScript, kOSAModeNull, &resultID);
			error = ::OSADispose(componentInstance, scriptID);
			error = ::OSADispose(componentInstance, resultID);
		}
	}
}

//----------------------------------------------------------------------------------------
// CScriptAction::DoIt
//----------------------------------------------------------------------------------------

void CScriptAction::DoIt()
{
	if (fScriptHandle != NULL)
		LoadAndExecuteScript(fScriptHandle);
}

//----------------------------------------------------------------------------------------
// CScriptAction::Read
//----------------------------------------------------------------------------------------

void* CScriptAction::Read(FW_CReadableStream& stream, FW_ClassTypeConstant type)
{
FW_UNUSED(type);
	FW_SOMEnvironment ev;
	return new CScriptAction(ev, stream);
}

//----------------------------------------------------------------------------------------
// CScriptAction::Write
//----------------------------------------------------------------------------------------

void CScriptAction::Write(FW_CWritableStream& stream, FW_ClassTypeConstant type, const void *object)
{
FW_UNUSED(type);
	FW_SOMEnvironment ev;
	CScriptAction* action = (CScriptAction*) object;
	action->ExternalizeToStream(ev, stream);
}

