//========================================================================================
//
//	File:				Part.h
//	Release Version:	$ ODF 2 $
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef PART_H
#define PART_H

// ----- Framework Layer -----

#ifndef FWPART_H
#include "FWPart.h"
#endif

// ----- OS Layer -----

#ifndef FWMNUITM_H
#include "FWMnuItm.h"
#endif

#ifndef FWTIME_H
#include "FWTime.h"
#endif

//========================================================================================
// Classes used by this interface
//========================================================================================

class CClockContent;

//========================================================================================
// Constants
//========================================================================================

const short kAnalogClock = 1;
const short kDigitalClock = 2;

//========================================================================================
// CClockPart
//========================================================================================

class CClockPart : public FW_CPart
{
public:
	FW_DECLARE_AUTO(CClockPart)

//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	CClockPart(ODPart* odPart);
	virtual void Initialize(Environment* ev, ODStorageUnit* storageUnit, FW_Boolean fromStorage);

	virtual ~ CClockPart();

//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	// ----- Factory Methods -----	
	virtual FW_CFrame* 		NewFrame(Environment* ev, ODFrame* odFrame, FW_CPresentation* presentation, FW_Boolean fromStorage);
	virtual FW_CContent* 	NewPartContent(Environment* ev);
	
	virtual FW_Handled		DoIdle(Environment* ev, const FW_CNullEvent& theNullEvent);
	
	virtual FW_Handled 		DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent);
	virtual FW_Handled 		DoAdjustMenus(Environment* ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot);
	virtual FW_Handled 		DoAbout(Environment* ev);
	
//----------------------------------------------------------------------------------------
//	New API
//
public:	
	void 					PlayTickSound();
	void 					PlayChimeSound();
	
	FW_CPresentation*		SwitchPresentation(FW_CPresentation* currentPresentation) const;
	FW_Boolean				IsAnalog(FW_CPresentation* presentation) const;

	short					Tick(Environment* ev, FW_CPresentation* presentation, const FW_CTime& currentTime);
	void					SynchContainingPartProperties(Environment* ev, FW_CPresentation* presentation);
	
	FW_Boolean				UseContainerColor() const;
	
	void					SetClock(Environment* ev, long hourOffset, const FW_CString& faceString);

private:
	void					OpenClockSettingsDialog(Environment* ev);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	FW_CTime 			fLastTime;
	CClockContent*		fClockContent;	
	FW_Boolean			fLockIdle;			// Use to block idle while changing clock type	
	FW_CIdler*			fIdler;
	
	FW_CPresentation*	fAnalogPresentation;	
	FW_CPresentation*	fDigitalPresentation;	
	FW_CPresentation*	fClockSettingsPresentation;	

#ifdef FW_BUILD_MAC
	FW_PlatformHandle 	fChimeSound;
	FW_PlatformHandle 	fTickSound;
#endif

	FW_Boolean			fUseContainerColor;
};

//----------------------------------------------------------------------------------------
// CClockPart::UseContainerColor
//----------------------------------------------------------------------------------------

inline FW_Boolean CClockPart::UseContainerColor() const
{
	return fUseContainerColor;
}

//----------------------------------------------------------------------------------------
// CClockPart::IsAnalog
//----------------------------------------------------------------------------------------

inline FW_Boolean CClockPart::IsAnalog(FW_CPresentation* presentation) const
{
	return presentation == fAnalogPresentation;
}

#endif
