//========================================================================================
//
//	File:				View.h
//	Release Version:	$ ODF 2 $
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef VIEW_H
#define VIEW_H

// ----- Part Layer -----

#ifndef FWSVIEW_H
#include "FWSView.h"
#endif

//========================================================================================
// Forward Declaration
//========================================================================================

class FW_CMouseEvent;
class FW_CEmbeddedMouseEvent;
class FW_CBorderMouseEvent;
class FW_CVirtualKeyEvent;

class CContainerPart;
class CContainerFrame;
class CPartContent;

//========================================================================================
// Class CContainerView
//========================================================================================

class CContainerView : public FW_CSuperView
{
public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
// Initialization/destruction
//
public:		
	CContainerView(Environment* ev, 
					CContainerFrame* frame, 
					FW_CRect& bounds, 
					FW_CPoint& extent,
					CContainerPart* part);
	virtual ~ CContainerView();
	
private:
	CContainerView(Environment* ev);
	
//----------------------------------------------------------------------------------------
// Inherited API
//
public:		
	// ----- Cursor -----
	virtual FW_Handled		AdjustCursor(Environment* ev, ODFacet* facet, const FW_CPoint& where, ODEventInfo* eventInfo);
	
	// ----- Drawing -----
	virtual void 			Draw(Environment* ev, ODFacet* odFacet, ODShape* invalidShape);
	virtual void 			AdjustToNewLayout(Environment *ev, 
											const FW_CPoint& oldExtent, 
											const FW_CPoint& newExtent, 
											FW_ERedrawVerb redraw);
	virtual void			SizeChanged(Environment* ev, const FW_CPoint& oldSize);

	// ----- Events -----
	virtual FW_Handled		DoMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	virtual FW_Handled		DoVirtualKey(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent);

	virtual FW_Boolean		WantsToBeTarget(Environment* ev);

	// ----- Archiving -----
	static void*			Create(FW_CReadableStream& stream, FW_ClassTypeConstant type);
	static void				Destroy(void* object, FW_ClassTypeConstant type);
	virtual void			Flatten(Environment* ev, FW_CWritableStream& stream) const;
	virtual void			InitializeFromStream(Environment* ev, FW_CReadableStream& stream);

//----------------------------------------------------------------------------------------
// Data Members
//
private:	
	CContainerPart*		fContainerPart;
	CPartContent*		fPartContent;
	CContainerFrame*	fContainerFrame;
};

#endif
