//========================================================================================
//
//	File:				ClipCmds.h
//	Release Version:	$ ODF 1 $
//
//	Author:				Henri Lamiraux
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef CLIPCMDS_H
#define CLIPCMDS_H

// ----- Part Layer -----

#ifndef FWCLPCMD_H
#include "FWClpCmd.h"
#endif

//========================================================================================
// Forward Declaration
//========================================================================================

class CDrawPart;
class CDrawFrame;
class CDrawSelection;
class CDrawSubscribeLink;
class CDrawLinkManager;
class CBaseShape;
class CDrawUndoContent;
class CDrawPublishLinkCollection;

//========================================================================================
// CDrawClipboardCommand
//========================================================================================

class CDrawClipboardCommand : public FW_CClipboardCommand
{
public:

  	FW_DECLARE_AUTO(CDrawClipboardCommand)

	CDrawClipboardCommand(Environment* ev, 
						 ODCommandID commandID,
						 CDrawPart* part, 
						 CDrawFrame* frame, 
						 CDrawSelection* selection,
						 FW_Boolean canUndo);

	virtual ~ CDrawClipboardCommand();

	// --- FW_CCommand overrides
	virtual void 		UndoIt(Environment* ev);
	virtual void 		RedoIt(Environment* ev);
	
	
	virtual void 		SaveUndoState(Environment* ev);
	virtual void 		SaveRedoState(Environment* ev);
	virtual void 		FreeUndoState(Environment* ev);
	virtual void 		FreeRedoState(Environment* ev);
	virtual void 		PropagateChanges(Environment* ev,  ODUpdateID id = kODUnknownUpdate);

	// ----- FW_CClipboardCommand overrides -----
	virtual void		PreCommand(Environment* ev);
	virtual void 		CommandDone(Environment* ev);

	

private:
	CDrawLinkManager* 	GetDrawLinkManager(Environment* ev) const;
	FW_Boolean 			IsPasteWithoutLink(Environment* ev) const;

private:
	CDrawPart*			fDrawPart;
	CDrawSelection*		fDrawSelection;
	CDrawUndoContent*	fUndoContent;
	CDrawSubscribeLink*	fSavedLink;
	CDrawPublishLinkCollection* fLinkSources;
};

#endif
