//========================================================================================
//
//	File:				DrawCont.cpp
//	Release Version:	$ ODF 2 $
//
//	Author:				Mary Boetcher
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "ODFDraw.hpp"

#ifndef DRAWCONT_H
#include "DrawCont.h"
#endif

#ifndef DRAWPART_H
#include "DrawPart.h"
#endif

#ifndef DRAWCLIP_H
#include "DrawClip.h"
#endif

#ifndef BASESHP_H
#include "BaseShp.h"
#endif

#ifndef GROUPSHP_H
#include "GroupShp.h"
#endif

#ifndef DRAWPRXY_H
#include "DrawPrxy.h"
#endif

#ifndef UTILS_H
#include "Utils.h"
#endif

#ifndef DRAWLINK_H
#include "DrawLink.h"
#endif

#ifndef DRWPRMSE_H
#include "DrwPrmse.h"
#endif

#ifndef DRAWSEL_H
#include "DrawSel.h"
#endif

// ----- Part Layer -----

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWPRESEN_H
#include "FWPresen.h"
#endif

#ifndef FWKIND_H
#include "FWKind.h"
#endif

// ----- OS Layer -----

#ifndef FWRECT_H
#include "FWRect.h"
#endif

#ifndef FWODGEOM_H
#include "FWODGeom.h"
#endif

#ifndef FWSUSINK_H
#include "FWSUSink.h"
#endif

#ifndef FWBARRAY_H
#include "FWBArray.h"
#endif

#ifndef FWFILEAC_H
#include "FWFileAc.h"
#endif

#ifndef SLMixOS_H
#include "SLMixOS.h"
#endif

// ----- Foundation Layer -----

#ifndef FWSTREAM_H
#include "FWStream.h"
#endif

#ifndef FWSUSINK_H
#include "FWSUSink.h"
#endif

#ifndef FWMEMORY_H
#include "FWMemory.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_ODTranslation_xh
#include <Translt.xh>
#endif

#ifndef SOM_ODShape_xh
#include <Shape.xh>
#endif

#ifndef SOM_ODStorageUnit_xh
#include <StorageU.xh>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

//========================================================================================
//	Runtime Information
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment odfdraw
#endif

FW_DEFINE_AUTO(CDrawPartContent)
FW_DEFINE_AUTO(CDrawSelectionContent)
FW_DEFINE_AUTO(CDrawUndoContent)
FW_DEFINE_AUTO(CDrawPromiseContent)
FW_DEFINE_AUTO(CDrawLinkContent)
FW_DEFINE_AUTO(CDrawLinkSourceContent)

//========================================================================================
//	class CDrawPartContent
//========================================================================================

//----------------------------------------------------------------------------------------
//	CDrawPartContent::CDrawPartContent
//----------------------------------------------------------------------------------------
//	CDrawPartContent constructor

CDrawPartContent::CDrawPartContent(Environment* ev, CDrawPart* part) :
	CDrawContent(ev, part, FW_kZeroRect) // Size doesn't matter
{
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
//	CDrawPartContent::~CDrawPartContent
//----------------------------------------------------------------------------------------
//	CDrawPartContent destructor

CDrawPartContent::~CDrawPartContent()
{
	FW_START_DESTRUCTOR
	
	if (fShapeList)
	{
		CBaseShape* shape;
		while ((shape = fShapeList->First()) != NULL)
		{
			fShapeList->Remove(shape);
			delete shape;
		}
		fProxyShapeCount = 0;
	}
}

//----------------------------------------------------------------------------------------
//	CDrawPartContent::PostInternalizeShape
//----------------------------------------------------------------------------------------

void CDrawPartContent::PostInternalizeShape(Environment* ev, const FW_CPoint& offset, CBaseShape* shape, short index)
{
	shape->OffsetShape(ev, -offset.x, -offset.y);
	shape->SetExternalizationIndex(index);	// so links can find their shapes
}

#if 0
//----------------------------------------------------------------------------------------
//	CDrawPartContent::SingleEmbeddedFrameInternalized
//----------------------------------------------------------------------------------------

void CDrawPartContent::SingleEmbeddedFrameInternalized(Environment* ev, 
													  FW_CEmbeddingFrame* scopeFrame,
													  ODPart* embeddedPart, 
													  ODFrame* embeddedFrame,
													  ODShape* suggestedShape,
													  ODTypeToken viewType)
{
	CProxyShape* insertedShape = AddSingleEmbeddedFrame(ev, 
														scopeFrame,
														embeddedPart, 
														embeddedFrame, 
														suggestedShape, 
														viewType);

	FW_CAcquiredODShape aqTempShape = ::FW_NewODShape(ev);
	insertedShape->GetUpdateBox(ev, aqTempShape);

	FW_CPresentation* presentation = fDrawPart->GetMainPresentation();
	CDrawFacetClipper facetClipper(fDrawPart);
	facetClipper.Clip(ev, presentation, aqTempShape);

	presentation->Invalidate(ev, aqTempShape);
}
#endif

//----------------------------------------------------------------------------------------
//	CDrawPartContent::MoveBackward
//----------------------------------------------------------------------------------------

FW_Boolean CDrawPartContent::MoveBackward(Environment* ev)
{
	FW_Boolean didIt = FALSE;

	CBaseShape* shape = fShapeList->First();
	CBaseShape* previous = NULL;
	while (shape != NULL)
	{
		if (shape->IsSelectedShape() && previous != NULL && !previous->IsSelectedShape())
		{
			fShapeList->Remove(shape);
			fShapeList->AddBefore(previous, shape);			
			shape->MovedBefore(ev, previous);
			shape = previous;			
			didIt = TRUE;
		}
		else
			previous = shape;
		
		shape = fShapeList->After(shape);
	}

	return didIt;
}

//----------------------------------------------------------------------------------------
//	CDrawPartContent::MoveForward
//----------------------------------------------------------------------------------------

FW_Boolean CDrawPartContent::MoveForward(Environment* ev)
{
	FW_Boolean didIt = FALSE;
	
	CBaseShape* shape = fShapeList->Last();
	CBaseShape* next = NULL;
	while (shape != NULL)
	{
		if (shape->IsSelectedShape() && next != NULL && !next->IsSelectedShape())
		{
			fShapeList->Remove(shape);
			fShapeList->AddAfter(next, shape);			
			shape->MovedAfter(ev, next);
			shape = next;			
			didIt = TRUE;
		}
		else
			next = shape;
			
		shape = fShapeList->Before(shape);
	}
		
	return didIt;
}

//----------------------------------------------------------------------------------------
//	CDrawPartContent::MoveToBack
//----------------------------------------------------------------------------------------

void CDrawPartContent::MoveToBack(Environment* ev)
{
	CShapeCollection extract;
	ExtractSelection(ev, &extract);
	
	CShapeCollectionIterator ite(&extract);
	for (CBaseShape* shape = ite.Last(); ite.IsNotComplete(); shape = ite.Previous())
	{
		fShapeList->AddFirst(shape);
		shape->MovedFirst(ev);
	}
}

//----------------------------------------------------------------------------------------
//	CDrawPartContent::MoveToFront
//----------------------------------------------------------------------------------------

void CDrawPartContent::MoveToFront(Environment* ev)
{
	CShapeCollection extract;
	ExtractSelection(ev, &extract);
	
	CShapeCollectionIterator ite(&extract);
	for (CBaseShape* shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		fShapeList->AddLast(shape);
		shape->MovedLast(ev);
	}
}

//----------------------------------------------------------------------------------------
//	CDrawPartContent::ExtractSelection
//----------------------------------------------------------------------------------------

void CDrawPartContent::ExtractSelection(Environment* ev, CShapeCollection* extract)
{
FW_UNUSED(ev);
	// ----- Create the extract list -----
	CShapeCollectionIterator ite(fShapeList);
	CBaseShape* shape;
	for (shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		if (shape->IsSelectedShape())
			extract->AddLast(shape);
	}
		
	// ----- Remove then from the shape list -----
	CShapeCollectionIterator ite2(extract);
	for (shape = ite2.First(); ite2.IsNotComplete(); shape = ite2.Next())
	{
		fShapeList->Remove(shape);
	}
}

//----------------------------------------------------------------------------------------
//	CDrawPartContent::NewShape
//----------------------------------------------------------------------------------------

CBaseShape* CDrawPartContent::NewShape(Environment* ev, unsigned short shapeType)
{	
FW_UNUSED(ev);
	CBaseShape *theShape = NULL;

	FW_ASSERT(shapeType != kProxyShape);		// We should never create a proxyShape this way
	
	switch (shapeType)
	{
		case kLineShape:
			theShape = FW_NEW(CLineShape, (fDrawPart));
			break;
			
		case kTextShape:
			theShape = FW_NEW(CTextShape, (fDrawPart));
			break;
		
		case kRectShape:
			theShape = FW_NEW(CRectShape, (fDrawPart));
			break;
			
		case kOvalShape:
			theShape = FW_NEW(COvalShape, (fDrawPart));
			break;
			
		case kRRectShape:
			theShape = FW_NEW(CRoundRectShape, (fDrawPart));
			break;
	}
	
	return theShape;
}

//----------------------------------------------------------------------------------------
//	CDrawPartContent::NewPromise
//----------------------------------------------------------------------------------------

FW_CPromise* CDrawPartContent::NewPromise(	Environment* ev, 
											ODStorageUnit* su, 
											FW_StorageKinds storageKind, 
											FW_CCloneInfo* cloneInfo)
{
	// Part caontent can only be asked to promise itself  in CloneInto, in which case it's necessary
	// first test for existence of kODPropContentFrame
	
	if (!su->Exists(ev, kODPropContentFrame, (ODValueType)NULL, 0))
		return NULL;
		
	// The promise resolver in OpenDoc seems to choke when trying to fulfill a promise made to a
	// link during a drop.  For a link of intrinsic content, the net result (besides an assertion failure
	// in debug version of openDoc) is that the destination doesn't update.  But when the promise was
	// written in the cloneInto method of a single embedded frame, the resultant failure inevitably 
	// leads to a crash.  The difference of is that in the first case, as soon as the link dest attempts
	// to read the content, the operation fails, leaving the destination intact.  In the single frame
	// case, the container, which owns the link dest, has no problem reading the embedded frame.  The 
	// failure occurs as the new embedded part attempts to initialize from storage.  This apparently leaves
	// the container in a bad state, so it crashes the next time an update is attempted.
	
	// for now, we'll avoid that situation:
	if (storageKind == FW_kLinkStorage)
		return NULL;
	
	// ----- Make a copy of myself -----
	CDrawPromiseContent* promisedContent = FW_NEW(CDrawPromiseContent, (ev, fDrawPart, this));

	// ----- Create a promise object -----
	return new CDrawPromise(ev, storageKind, cloneInfo, promisedContent);
}


//========================================================================================
//	class CDrawSelectionContent
//========================================================================================

//----------------------------------------------------------------------------------------
//	CDrawSelectionContent::CDrawSelectionContent
//----------------------------------------------------------------------------------------
//	CDrawSelectionContent constructor

CDrawSelectionContent::CDrawSelectionContent(Environment* ev, CDrawPart* part, CDrawSelection* selection) :	
	CDrawContent(ev, part, selection->GetDragRect()),
	fDrawSelection(selection)
{
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
//	CDrawSelectionContent::~CDrawSelectionContent
//----------------------------------------------------------------------------------------
//	CDrawSelectionContent destructor

CDrawSelectionContent::~CDrawSelectionContent()
{
	FW_START_DESTRUCTOR
}

//----------------------------------------------------------------------------------------
//	CDrawSelectionContent::NewPromise
//----------------------------------------------------------------------------------------

FW_CPromise* CDrawSelectionContent::NewPromise(Environment* ev, ODStorageUnit* su, FW_StorageKinds storageKind, FW_CCloneInfo* cloneInfo)
{
FW_UNUSED(su);

	// update our content rect before copying self for promise.
	fContentRect = fDrawSelection->GetDragRect();
	
	// ----- Make a copy of myself -----
	CDrawPromiseContent* promisedContent = FW_NEW(CDrawPromiseContent, (ev, fDrawPart, this));

	// ----- Create a promise object -----
	return new CDrawPromise(ev, storageKind, cloneInfo, promisedContent);
}

//----------------------------------------------------------------------------------------
//	CDrawSelectionContent::SingleEmbeddedFrameInternalized
//----------------------------------------------------------------------------------------

void CDrawSelectionContent::SingleEmbeddedFrameInternalized(Environment* ev, 
														  FW_CEmbeddingFrame* scopeFrame,
														  ODPart* embeddedPart, 
														  ODFrame* embeddedFrame,
														  ODShape* suggestedShape,
														  ODTypeToken viewType)
{
	fDrawSelection->CloseSelection(ev);
	CProxyShape* insertedShape = AddSingleEmbeddedFrame(ev, 
														scopeFrame, 
														embeddedPart, 
														embeddedFrame, 
														suggestedShape, 
														viewType);	
	PostInternalizeShape(ev, FW_kZeroPoint, insertedShape, 1);										
}

//----------------------------------------------------------------------------------------
//	CDrawSelectionContent::PostInternalizeShape
//----------------------------------------------------------------------------------------

void CDrawSelectionContent::PostInternalizeShape(Environment* ev, const FW_CPoint& offset, CBaseShape* shape, short index)
{
FW_UNUSED(offset);
	shape->SetExternalizationIndex(index);	// so links can find their shapes

	// ----- Add the shape to the right places -----
	fDrawPart->AddShapeToPart(ev, shape);	// Add to the part data
	
	// ----- Shape has already been added to the selection list (fShapeList)
	fDrawSelection->ShapeAdded(ev, shape);
}

//----------------------------------------------------------------------------------------
//	CDrawSelectionContent::AcquireSuggestedFrameShape
//----------------------------------------------------------------------------------------

ODShape* CDrawSelectionContent::AcquireSuggestedFrameShape(Environment* ev)
{
	FW_CRect selectionRect = fDrawSelection->GetDragRect();
	selectionRect.Place(FW_kZeroPoint);
	
	return ::FW_NewODShape(ev, selectionRect);
}

//========================================================================================
//	class CDrawUndoContent
//========================================================================================

//----------------------------------------------------------------------------------------
//	CDrawUndoContent::CDrawUndoContent
//----------------------------------------------------------------------------------------
//	CDrawUndoContent constructor

CDrawUndoContent::CDrawUndoContent(Environment* ev, CDrawPart* part, CDrawSelection* selection) :
	CDrawContent(ev, part, selection->GetDrawContent(ev)),
	fDrawSelection(selection)
{
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
//	CDrawUndoContent::~CDrawUndoContent
//----------------------------------------------------------------------------------------
//	CDrawUndoContent destructor

CDrawUndoContent::~CDrawUndoContent()
{
	FW_START_DESTRUCTOR
}

//----------------------------------------------------------------------------------------
//	CDrawUndoContent::RemoveShapeSelection
//----------------------------------------------------------------------------------------

void CDrawUndoContent::RemoveShapeSelection(Environment *ev)
{
	// Select the saved shapes
	fDrawSelection->SelectContent(ev, this);

	// Clear the selected shapes
	fDrawSelection->ClearSelection(ev);

}

//----------------------------------------------------------------------------------------
//	CDrawUndoContent::RestoreShapeSelection
//----------------------------------------------------------------------------------------

void CDrawUndoContent::RestoreShapeSelection(Environment *ev)
{
	// Add the saved shapes back into the part
	CDrawContentShapeIterator ite(this);
	for (CBaseShape* shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		shape->RestoreShape(ev);
	}

	// Redraw the restored shapes
	RedrawShapes(ev);

	// Select the saved shapes
	fDrawSelection->SelectContent(ev, this);
}

//----------------------------------------------------------------------------------------
//	CDrawUndoContent::DeleteSavedShapes
//----------------------------------------------------------------------------------------

void CDrawUndoContent::DeleteSavedShapes(Environment *ev)
{
	CBaseShape* shape;
	
	// Before deleting the shapes, make sure they are not promised
	CDrawContentShapeIterator ite(this);
	for (shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		shape->CheckPromise(ev);
	}
	
	// ----- Delete all the saved shapes -----
	while ((shape = GetFirstShape()) != NULL)
	{
		RemoveShape(ev, shape);
		delete shape;
	}		
}

//----------------------------------------------------------------------------------------
//	CDrawUndoContent::RemoveFromLinks
//----------------------------------------------------------------------------------------
void CDrawUndoContent::RemoveFromLinks(Environment* ev)
{
	// Remove the saved shapes from any link sources they're in
	CDrawContentShapeIterator it(this);
	for (CBaseShape* shape = it.First(); it.IsNotComplete(); shape = it.Next())
	{
		CDrawPublishLink* linkSource = shape->GetPublishLink();
		if (linkSource)
		{
			linkSource->RemoveShape(ev, shape);
		}

		CDrawSubscribeLink* link = shape->GetSubscribeLink();
		if (link)
		{
			link->Unregister(ev);
			fDrawPart->GetLinkManager(ev)->RemoveFromDestLinkList(ev, link);
		}
	}
}

//----------------------------------------------------------------------------------------
//	CDrawUndoContent::RestoreLinks
//----------------------------------------------------------------------------------------
void CDrawUndoContent::RestoreLinks(Environment* ev)
{
	// Add the saved shapes back into any link sources they were in
	CDrawContentShapeIterator it(this);
	for (CBaseShape* shape = it.First(); it.IsNotComplete(); shape = it.Next())
	{
		CDrawPublishLink* linkSource = shape->GetPublishLink();
		if (linkSource)
		{
			linkSource->AddShape(ev, shape);
		}
		
		CDrawSubscribeLink* link = shape->GetSubscribeLink();
		if (link)
		{
			link->Register(ev, fDrawPart);
			fDrawPart->GetLinkManager(ev)->AddToDestLinkList(ev, link);
		}
	}
}

//----------------------------------------------------------------------------------------
//	CDrawUndoContent::DeleteEmptyLinkSources
//----------------------------------------------------------------------------------------
void CDrawUndoContent::DeleteEmptyLinkSources(Environment* ev)
{
	// Check for link sources left empty because all their shapes were removed
	FW_CLinkManager* linkMgr = fDrawPart->GetLinkManager(ev);

	CDrawContentShapeIterator it(this);
	for (CBaseShape* shape = it.First(); it.IsNotComplete(); shape = it.Next())
	{
		CDrawPublishLink* linkSource = shape->GetPublishLink();
		if (linkSource && linkSource->IsEmpty())
		{
			linkMgr->BreakSourceLink(ev, linkSource);
			delete linkSource;
		}
	}
}

//----------------------------------------------------------------------------------------
//	CDrawUndoContent::DeleteSavedShapesAndLinks
//----------------------------------------------------------------------------------------
void CDrawUndoContent::DeleteSavedShapesAndLinks(Environment* ev)
{
	CBaseShape* shape;
	
	// Before deleting the shapes, make sure they are not promised
	CDrawContentShapeIterator ite(this);
	for (shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
	{
		shape->CheckPromise(ev);
	}
	
	// ----- Delete all the saved shapes -----
	while ((shape = GetFirstShape()) != NULL)
	{
		RemoveShape(ev, shape);
		// Check for an orphaned link and delete it
		CDrawSubscribeLink* link = shape->GetSubscribeLink();
		if (link)
			delete link;	// deletes the shape
		else
			delete shape;
	}		
}

//========================================================================================
//	class CDrawPromiseContent
//========================================================================================

//----------------------------------------------------------------------------------------
//	CDrawPromiseContent::CDrawPromiseContent
//----------------------------------------------------------------------------------------
//	CDrawPromiseContent constructor

CDrawPromiseContent::CDrawPromiseContent(Environment* ev, 
										CDrawPart* part, 
										CDrawContent* content) :
	CDrawContent(ev, part, content)
{
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
//	CDrawPromiseContent::~CDrawPromiseContent
//----------------------------------------------------------------------------------------
//	CDrawPromiseContent destructor

CDrawPromiseContent::~CDrawPromiseContent()
{
	FW_START_DESTRUCTOR
}

//----------------------------------------------------------------------------------------
//	CDrawPromiseContent::IsOKtoWrite
//----------------------------------------------------------------------------------------

FW_Boolean CDrawPromiseContent::IsOKtoWrite(Environment* ev, CBaseShape* shape)
{
FW_UNUSED(ev);
FW_UNUSED(shape);
	return TRUE;
}

//----------------------------------------------------------------------------------------
//	CDrawPromiseContent::PromiseShape
//----------------------------------------------------------------------------------------

void CDrawPromiseContent::PromiseShape(Environment* ev, FW_StorageKinds storageKind, FW_Boolean state)
{
FW_UNUSED(ev);
	CDrawContentShapeIterator ite(this);
	for (CBaseShape *shape = ite.First(); ite.IsNotComplete(); shape = ite.Next())
		shape->Promised(storageKind, state);
}


//========================================================================================
//	class CDrawLinkSourceContent
//========================================================================================

//----------------------------------------------------------------------------------------
//	CDrawLinkSourceContent::CDrawLinkSourceContent
//----------------------------------------------------------------------------------------
//	CDrawLinkSourceContent constructor

CDrawLinkSourceContent::CDrawLinkSourceContent(	Environment* ev, 
												CDrawPart* part, 
												CDrawContent* selectedContent,
												CDrawPublishLink* linkSrc) :
	CDrawPromiseContent(ev, part, selectedContent),
	fLinkSource(linkSrc)
{
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
//	CDrawLinkSourceContent::~CDrawLinkSourceContent
//----------------------------------------------------------------------------------------
//	CDrawLinkSourceContent destructor

CDrawLinkSourceContent::~CDrawLinkSourceContent()
{
	FW_START_DESTRUCTOR
}

//----------------------------------------------------------------------------------------
//	CDrawLinkSourceContent::NewPromise
//----------------------------------------------------------------------------------------
FW_CPromise* CDrawLinkSourceContent::NewPromise(Environment* ev,  ODStorageUnit* su, FW_StorageKinds storageKind,  FW_CCloneInfo* cloneInfo)//Override
{
FW_UNUSED(su);

	// ----- Create a promise object -----
	return new CDrawPromise(ev, storageKind, cloneInfo, this, fLinkSource);
}


//========================================================================================
//	class CDrawLinkContent
//========================================================================================

//----------------------------------------------------------------------------------------
//	CDrawLinkContent::CDrawLinkContent
//----------------------------------------------------------------------------------------
//	CDrawLinkContent constructor

CDrawLinkContent::CDrawLinkContent(Environment* ev, CDrawPart* part, CDrawSubscribeLink* link) :
	CDrawContent(ev, part, FW_kZeroRect),
	fLink(link)
{
	// This constructor is called when a new destination link is created
}

//----------------------------------------------------------------------------------------
//	CDrawLinkContent::CDrawLinkContent
//----------------------------------------------------------------------------------------
//	CDrawLinkContent constructor

CDrawLinkContent::CDrawLinkContent(Environment* ev, CDrawSubscribeLink* link, CDrawPart* part, CDrawLinkContent* content)
:	CDrawContent(ev, part, content),
	fLink(link)
{
	// This constructor is called when a link is updated (not the first time)
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
//	CDrawLinkContent::~CDrawLinkContent
//----------------------------------------------------------------------------------------
//	CDrawLinkContent destructor

CDrawLinkContent::~CDrawLinkContent()
{
	FW_START_DESTRUCTOR
}

//----------------------------------------------------------------------------------------
//	CDrawLinkContent::GroupShapes
//----------------------------------------------------------------------------------------
void CDrawLinkContent::GroupShapes(Environment* ev, CGroupShape* group)
{
FW_UNUSED(ev);
	// Add our shapes to the group shape
	CDrawContentShapeIterator it(this);
	for (CBaseShape* shape = it.First(); it.IsNotComplete(); shape = it.Next())
	{
		group->AddShape(shape);
	}
}

//----------------------------------------------------------------------------------------
//	CDrawLinkContent::SingleEmbeddedFrameInternalized
//----------------------------------------------------------------------------------------

void CDrawLinkContent::SingleEmbeddedFrameInternalized(Environment* ev, 
													   FW_CEmbeddingFrame* scopeFrame,
													   ODPart* embeddedPart, 
													   ODFrame* embeddedFrame,
													   ODShape* suggestedShape,
													   ODTypeToken viewType)
{
	CProxyShape* newShape = AddSingleEmbeddedFrame(ev, 
												   scopeFrame, 
												   embeddedPart, 
												   embeddedFrame, 
												   suggestedShape, 
												   viewType);
	PostInternalizeShape(ev, FW_kZeroPoint, newShape, 1);												
}

