//========================================================================================
//
//	File:				DrawCont.h
//	Release Version:	$ ODF 2 $
//
//	Author:				Mary Boetcher
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef DRAWCONT_H
#define DRAWCONT_H

#ifndef CONTENT_H
#include "Content.h"
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class CDrawPart;
class CDrawSelection;
class CDrawPromise;
class CDrawSubscribeLink;
class CGroupShape;

//========================================================================================
//	class CDrawPartContent
//========================================================================================

class CDrawPartContent : public CDrawContent
{
public:
  
  	FW_DECLARE_AUTO(CDrawPartContent)
  
	CDrawPartContent(Environment* ev, CDrawPart* part);
	virtual ~CDrawPartContent();

	virtual void		PostInternalizeShape(Environment* ev, 
									 const FW_CPoint& offset, 
									 CBaseShape* shape, 
									 short index);

	CBaseShape*			NewShape(Environment* ev, unsigned short shapeType);

	FW_Boolean			MoveForward(Environment *ev);
	FW_Boolean			MoveBackward(Environment *ev);
	void				MoveToFront(Environment *ev);
	void				MoveToBack(Environment *ev);
	void				ExtractSelection(Environment* ev, CShapeCollection* extract);
	virtual FW_CPromise* NewPromise(Environment* ev, 
							ODStorageUnit* su, 
							FW_StorageKinds storageKind, 
							FW_CCloneInfo* cloneInfo);

};

//========================================================================================
//	class CDrawSelectionContent
//========================================================================================

class CDrawSelectionContent : public CDrawContent
{
public:
  
  	FW_DECLARE_AUTO(CDrawSelectionContent)
  
	CDrawSelectionContent(Environment* ev, 
						CDrawPart* part, 
						CDrawSelection* selection);
	virtual ~CDrawSelectionContent();

	virtual void 		PostInternalizeShape(Environment* ev, 
									const FW_CPoint& offset, 
									CBaseShape* shape, 
									short index);

	virtual void 		SingleEmbeddedFrameInternalized(Environment* ev, 
									  FW_CEmbeddingFrame* scopeFrame,
									  ODPart* embeddedPart, 
									  ODFrame* embeddedFrame,
									  ODShape* suggestedShape,
									  ODTypeToken viewType);

	virtual FW_CPromise* NewPromise(Environment* ev, 
									  ODStorageUnit* su, 
									  FW_StorageKinds storageKind, 
									  FW_CCloneInfo* cloneInfo);

	virtual ODShape* 	AcquireSuggestedFrameShape(Environment* ev);

  private:
	CDrawSelection*		fDrawSelection;
};

//========================================================================================
//	class CDrawUndoContent
//========================================================================================

class CDrawUndoContent : public CDrawContent
{
public:
  
  	FW_DECLARE_AUTO(CDrawUndoContent)
  
	CDrawUndoContent(Environment* ev, CDrawPart* part, CDrawSelection* selection);
	virtual ~CDrawUndoContent();

	// ----- New API -----
	void 				RemoveShapeSelection(Environment *ev);
	void 				RestoreShapeSelection(Environment *ev);
	void 				DeleteSavedShapes(Environment *ev);

	void 				RemoveFromLinks(Environment* ev);
	void 				RestoreLinks(Environment* ev);
	void				DeleteEmptyLinkSources(Environment* ev);
	void				DeleteSavedShapesAndLinks(Environment* ev);

private:
	CDrawSelection*		fDrawSelection;
};

//========================================================================================
//	class CDrawPromiseContent
//========================================================================================

class CDrawPromiseContent : public CDrawContent
{
  public:
  
  	FW_DECLARE_AUTO(CDrawPromiseContent)
  
	CDrawPromiseContent(Environment* ev, 
										CDrawPart* part, 
										CDrawContent* content);
	virtual ~CDrawPromiseContent();

	virtual FW_Boolean 	IsOKtoWrite(Environment* ev, CBaseShape* shape);

	void				PromiseShape(Environment* ev, 
							FW_StorageKinds storageKind, 
							FW_Boolean state);

	FW_CRect			GetSelectedRect() const
							{return fSelectedRect;}
  private:
	CDrawPromise*	fPromise;
	FW_CRect		fSelectedRect;
};

//========================================================================================
//	class CDrawLinkSourceContent
//========================================================================================

class CDrawLinkSourceContent : public CDrawPromiseContent
{
  public:
  
  	FW_DECLARE_AUTO(CDrawLinkSourceContent)
  
	CDrawLinkSourceContent(	Environment* ev, 
												CDrawPart* part, 
												CDrawContent* selectedContent,
												CDrawPublishLink* linkSrc);
	virtual ~CDrawLinkSourceContent();

	
	virtual FW_CPromise* NewPromise(Environment* ev,  ODStorageUnit* su, FW_StorageKinds storageKind,  FW_CCloneInfo* cloneInfo); //Override

  private:
  	
	CDrawPublishLink* fLinkSource;
  	
};

//========================================================================================
//	class CDrawLinkContent
//========================================================================================

class CDrawLinkContent : public CDrawContent
{
  public:
  
  	FW_DECLARE_AUTO(CDrawLinkContent)
  
	CDrawLinkContent(Environment* ev, CDrawPart* part, CDrawSubscribeLink* link);
	CDrawLinkContent(Environment* ev, CDrawSubscribeLink* link, CDrawPart* part, CDrawLinkContent* content);
	virtual ~CDrawLinkContent();
										
	virtual void		SingleEmbeddedFrameInternalized(Environment* ev,
								FW_CEmbeddingFrame* scopeFrame,
								ODPart* odEmbeddedPart, 
								ODFrame* odEmbeddedFrame,
								ODShape* suggestedShape,
								ODTypeToken viewType);

	void				GroupShapes(Environment* ev, CGroupShape* group);

  private:
	CDrawSubscribeLink*	fLink;
};


#endif
