//========================================================================================
//	File:				Views.fr
//	Release Version:	$ ODF 2 $
//
//	Contains:			Resources defining the Form part's views
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//========================================================================================

#ifndef FWRESFIL_K
#include "FWResFil.k"
#endif

#ifndef FWVIEWS_FR
#include "FWViews.fr"
#endif

#ifndef FWNOTDEF_H
#include "FWNotDef.h"	 
#endif

#ifndef DEFINES_K
#include "Defines.k"
#endif

//========================================================================================
// Defines
//========================================================================================

// H & V are reference size for the Form frame.
// Their values are not important because all the views will be resized at runtime 
// to adjust to the real frame size defined by the part. What is important is the 
// placement of the subviews relative to this reference size and their binding flags.
// Note: in this example the bounds & bindings of RFormView, the content view, are not 
// important either because the frame takes care of it in CFormView::CenterInFrame() 

#define H	FW_FIX(600)
#define V	FW_FIX(1000)

// Fonts not defined in "FWViews.fr"
#define TIMES_12_BOLD		{ FW_FIX(12), FW_kBold, "Times" }

//========================================================================================
// type RFormView
//========================================================================================

type RFormView : FW_RPictSView(Label='Frmv')
{
	// This class adds a picture ID and 3 radio-clusters to FW_RSuperView
	// (See code in CFormView::InitializeFromStream()).
Picture2:
	integer;
	
	FW_RRadioCluster;	
	FW_RRadioCluster;	
	FW_RRadioCluster;
};

//========================================================================================
// type RScrollEdit
//========================================================================================
// Custom scrolling edit view 

type RScrollEdit : FW_REditView(Label='Sedv')
{
HorizViewID:
	longint;			// Horizontal scrollbar id, or 0 for no scrollbar
VertViewID:
	longint;			// Vertical scrollbar id, or 0 for no scrollbar
TextWidth:
	FW_RFixed;			// text width (or 0 to use same width as view)
};

//========================================================================================
// resource RFormFrame(kFormView)
//========================================================================================
// See constants in "FWViews.k", macros & resource types in "FWViews.fr"

resource FW_RFrameLayout(kFormView)
{
	{H,V},					// LayoutSize
	{						// Start list of frame's subviews
		RFormView
		(
			kFormViewID,				// view id
			{0, 0, H, V},				// bounds
			FW_kFitToEnclosure,			// bindings
			FW_POINT(560, 800),			// extent
			1,							// Make it the ContentView
			FW_kXYScrolling,			// scrolling direction
			{					// Start list of SubViews
				FW_REditView	// First name
				(
				    kFirstNameEdViewID,				// view id
					FW_BOUNDS(290, 90, 450, 107),	// bounds
					FW_kFixedBounds,				// binding
					20,								// max chars
					9,								// attributes
					TIMES_12_BOLD,
					""								// initial text
				),
				FW_REditView	// Last name
				(
					kLastNameEdViewID,				// view id
					FW_BOUNDS(290, 110, 450, 127),
					FW_kFixedBounds,				// binding
					20,								// max chars
					9,								// attributes
					TIMES_12_BOLD,
					""								// initial text
				),
				FW_REditView	// member name
				(
					kMemberNameEdViewID,			// view id
					FW_BOUNDS(290, 130, 450, 147),
					FW_kFixedBounds,				// binding
					20,								// max chars
					9,								// attributes
					TIMES_12_BOLD,
					""								// initial text
				),
				FW_REditView	// e-mail address
				(
					kEMailEdViewID,					// view id
					FW_BOUNDS(290, 150, 450, 167),
					FW_kFixedBounds,				// binding
					30,								// max chars
					9,								// attributes
					TIMES_12_BOLD,
					""								// initial text
				),
/**				
				FW_RScrollBar	// Horizontal scrollbar for text-edit
				(
					kEditHScrollBarID,			// view id
					{FW_FIX(30), FW_FIX(289), FW_FIX(450), FW_FIX(305)},	// bounds 
					FW_kFixedBounds,			// view binding
					0,							// control message
					0,							// control receiver
					0,							// control value
					0,							// SB Min Value (adjusted at runtime)
					1,							// SB Max Value (adjusted at runtime)
					FW_ONE,						// SB Minor Units (adjusted at runtime)
					FW_ONE						// SB Major Units (adjusted at runtime)
				),
**/
				FW_RScrollBar	// Vertical scrollbar for text-edit
				(
					kEditVScrollBarID,			// view id
					FW_BOUNDS(449, 200, 465, 290),
					FW_kFixedBounds,			// view binding
					0,							// control message
					0,							// control receiver
					0,							// control value
					0,							// SB Min Value (adjusted at runtime)
					1,							// SB Max Value (adjusted at runtime)
					FW_ONE,						// SB Minor Units (adjusted at runtime)
					FW_ONE						// SB Major Units (adjusted at runtime)
				),
				RScrollEdit		// Comments (custom scrolling edit view)
				(
					kCommentsEdViewID,				// view id
					FW_BOUNDS(30, 200, 450, 290),
					FW_kFixedBounds,				// binding
					-1,								// max chars (32K)
					13,								// attributes
					FW_NORMAL_FONT,
					"",								// initial text
					0, // kEditHScrollBarID,		// Horizontal SB
					kEditVScrollBarID,				// Vertical SB
					FW_FIX(600)						// text width (used only with horiz SB)
				),
				FW_RButton
				(
					kSubscribeCheckID,				// view id
					FW_BOUNDS(30, 310, 200, 324),
					FW_kFixedBounds,				// binding
					FW_kButtonPressedMsg,			// control message
					0,								// control receiver
					1,								// control value
					FW_kCheckButton,				// button kind
					TIMES_12_BOLD,
					"Subscribe to ODFFlash"			
				),
				FW_RGroupBox
				(
					0,								// view id
					FW_BOUNDS(30, 360, 220, 440),
					FW_kFixedBounds,				// binding
					TIMES_12_BOLD,
					FW_RGB_WHITE,					// erase color
					"Modem speed used"
				),
				FW_RButton
				(
					k14400RadioID,					// view id
					FW_BOUNDS(40, 380, 210, 396),
					FW_kFixedBounds,				// binding
					0,								// control message
					0,								// control receiver
					0,								// control value
					FW_kRadioButton,				// button kind
					FW_SYSTEM_FONT,
					"14.4 Kbps or lower"			
				),
				FW_RButton
				(
					k28800RadioID,					// view id
					FW_BOUNDS(40, 400, 210, 416),
					FW_kFixedBounds,				// binding
					0,								// control message
					0,								// control receiver
					1,								// control value
					FW_kRadioButton,				// button kind
					FW_SYSTEM_FONT,
					"28.8 Kbps to 256 Kbps"			
				),
				FW_RButton
				(
					kFasterRadioID,					// view id
					FW_BOUNDS(40, 420, 210, 436),
					FW_kFixedBounds,				// binding
					0,								// control message
					0,								// control receiver
					0,								// control value
					FW_kRadioButton,				// button kind
					FW_SYSTEM_FONT,
					"Faster than 256 Kbps"			
				),
				FW_RListBox
				(
					kPlatformListBoxID,				// view id
					FW_BOUNDS(250, 362, 400, 440),
					FW_kFixedBounds,				// binding
					10,								// number of rows
					1,								// vertical SB
					0,								// single selection
					1,								// use focus frame
					0,								// use clientdata
					FW_kListBoxDoubleClickedMsg,	// DblClick message
					FW_HELVETICA12,					// font
					{								// List of strings
						"Mac OS",
						"OS/2 Warp",
						"Windows 95",
						"Windows NT",
						"IBM AIX",
						"Sun Sparc Solaris",
						"Sun OS 4.1.x",
						"HP /UX",
						"Unix IRIX",
						"Unix Linux"
					}
				),
				FW_RButton
				(
					kAddButtonID,					// view id
					FW_BOUNDS(410, 370, 480, 390),
					FW_kFixedBounds,				// binding
					FW_kButtonPressedMsg,			// control message
					FW_kFrameReceiver,				// receiver
					0,								// control value
					FW_kPushButton,					// button kind
					FW_SYSTEM_FONT,
					"Add"			
				),
				FW_RButton
				(
					kRemoveButtonID,				// view id
					FW_BOUNDS(410, 400, 480, 420),
					FW_kFixedBounds,				// binding
					FW_kButtonPressedMsg,			// control message
					FW_kFrameReceiver,				// receiver
					0,								// control value
					FW_kPushButton,					// button kind
					FW_SYSTEM_FONT,
					"Remove"			
				),
				FW_RPopupMenu
				(
					kBrowseTimePopupID,				// view id
					FW_BOUNDS(30, 477, 230, 497),
					FW_kFixedBounds,				// binding
					FW_kPopupClickedMsg,			// control message
					FW_kFrameReceiver,				// control receiver
					0,								// control value
					kBrowseTimeMenuResID,			// Mac menu id
					0,								// title width
					2,								// Initial menu item
					0,								// Menu variation
					0,								// Menu refCon	
					FW_SYSTEM_FONT,
					""								// title
				),
				FW_RPopupMenu
				(
					kOnlineTimePopupID,				// view id
					FW_BOUNDS(30, 540, 230, 560),
					FW_kFixedBounds,				// binding
					FW_kPopupClickedMsg,			// control message
					0,								// control receiver
					0,								// control value
					kOnlineTimeMenuResID,			// Mac menu id
					0,								// title width
					1,								// Initial menu item
					0,								// Menu variation
					0,								// Menu refCon	
					FW_SYSTEM_FONT,
					""								// title
				),
				FW_RGroupBox
				(
					0,								// view id
					FW_BOUNDS(30, 580, 350, 700),
					FW_kFixedBounds,				// binding
					TIMES_12_BOLD,
					FW_RGB_WHITE,					// erase color
					" How did you discover ODFWired?"			
				),
				FW_RButton
				(
					kWordOfMouthRadioID,			// view id
					FW_BOUNDS(40, 600, 300, 616),
					FW_kFixedBounds,				// binding
					0,								// control message
					0,								// control receiver
					0,								// control value
					FW_kRadioButton,				// button kind
					FW_SYSTEM_FONT,
					"Word of mouth"			
				),
				FW_RButton
				(
					kHyperLinkRadioID,				// view id
					FW_BOUNDS(40, 620, 300, 636),
					FW_kFixedBounds,				// binding
					0,								// control message
					0,								// control receiver
					0,								// control value
					FW_kRadioButton,				// button kind
					FW_SYSTEM_FONT,
					"Hyperlink from another site:"			
				),
				FW_RButton
				(
					kMacTechRadioID,				// view id
					FW_BOUNDS(40, 640, 300, 656),
					FW_kFixedBounds,				// binding
					0,								// control message
					0,								// control receiver
					1,								// control value
					FW_kRadioButton,				// button kind
					FW_SYSTEM_FONT,
					"Ad in MacTech Magazine"			
				),
				FW_RButton
				(
					kMagazineRadioID,				// view id
					FW_BOUNDS(40, 660, 300, 676),
					FW_kFixedBounds,				// binding
					0,								// control message
					0,								// control receiver
					0,								// control value
					FW_kRadioButton,				// button kind
					FW_SYSTEM_FONT,
					"Other magazine, newspaper, radio"			
				),
				FW_RButton
				(
					kOtherRadioID,					// view id
					FW_BOUNDS(40, 680, 300, 696),
					FW_kFixedBounds,				// binding
					0,								// control message
					0,								// control receiver
					0,								// control value
					FW_kRadioButton,				// button kind
					FW_SYSTEM_FONT,
					"Other"			
				),
				FW_RButton
				(
					kYesRadioID,					// view id
					FW_BOUNDS(40, 725, 100, 740),
					FW_kFixedBounds,				// binding
					0,								// control message
					0,								// control receiver
					0,								// control value
					FW_kRadioButton,				// button kind
					FW_SYSTEM_FONT,
					"Yes"			
				),
				FW_RButton
				(
					kNoRadioID,						// view id
					FW_BOUNDS(40, 745, 100, 760),
					FW_kFixedBounds,				// binding
					FW_kButtonPressedMsg,			// control message
					FW_kFrameReceiver,				// control receiver
					0,								// control value
					FW_kRadioButton,				// button kind
					FW_SYSTEM_FONT,
					"No"			
				),	
				FW_RButton
				(
					kSubscribeButtonID,				// view id
					FW_BOUNDS(300, 750, 450, 775),
					FW_kFixedBounds,				// binding
					FW_kButtonPressedMsg,			// control message
					FW_kFrameReceiver,				// receiver
					0,								// control value
					FW_kPushButton,					// button kind
					FW_SYSTEM_FONT,
					"Join ODFWired!"			
				)
			},							// End list of Subviews

			kFormPict1,					// PictureID field of FW_RPictSView
			kFormPict2,					// PictureID field of RFormView
			
			// RFormView adds 3 radio-clusters to FW_RSuperView
			FW_RRadioCluster	
			(
				{ k14400RadioID, k28800RadioID, kFasterRadioID } // List of radios
			),
			FW_RRadioCluster	
			(
				{ kWordOfMouthRadioID, kHyperLinkRadioID, kMacTechRadioID,
					kMagazineRadioID, kOtherRadioID } // List of radios
			),
			FW_RRadioCluster	
			(
				{ kYesRadioID, kNoRadioID }			// List of radios
			) 			
	
		),							// End RFormView resource
		FW_RScrollBar		// Horizontal
		(
			kHorzScrollBarID,			// view id
			FW_STDHORIZONTALSB(H,V),	//  bounds 
			FW_kHScrollBarBinding,		// Standard horizontal SB binding
			0,							// control message
			0,							// control receiver
			0,							// control value
			0,							// SB Min Value (adjusted at runtime)
			1,							// SB Max Value (adjusted at runtime)
			FW_ONE,						// SB Minor Units (adjusted at runtime)
			FW_ONE						// SB Major Units (adjusted at runtime)
		),
		FW_RScrollBar		// Vertical
		(
			kVertScrollBarID,			// view id
			FW_STDVERTICALSB(H, V),		// bounds 
			FW_kVScrollBarBinding,		// Standard vertical SB binding
			0,							// control message
			0,							// control receiver
			0,							// control value
			0,							// SB Min Value (adjusted at runtime)
			1,							// SB Max Value (adjusted at runtime)
			FW_ONE,						// SB Minor Units (adjusted at runtime)
			FW_ONE						// SB Major Units (adjusted at runtime)
		),
		FW_RGrowBox
		(
			kGrowBoxID,					// view ID
			FW_STDGROWBOX(H,V),			// bounds
			FW_kGrowBoxBinding			// standard grow box binding
		)
	},							// End list of frame's subviews
	
	{						// Scroller
		FW_RScrollBarScroller
		(
			0,							// AutoScrollInset: not used here
			{FW_FIX(16), FW_FIX(16)},	// Scroll Units
			1,							// Responds to function key
			0,							// Live Scrolling
			kHorzScrollBarID,			// Horizontal scroll-bar ID
			kVertScrollBarID			// Vertical scroll-bar ID
		) 
	}
};

//========================================================================================
// type RPwdEditView
//========================================================================================
// Custom edit view to type hidden passwords

type RPwdEditView : FW_REditView(Label='Pwdv')
{
};

//========================================================================================
// resource FW_RFrameLayout(kPasswordDialog)
//========================================================================================
// See constants in "FWViews.k", macros & resource types in "FWViews.fr"

resource FW_RFrameLayout(kPasswordDialog)
{
	{FW_FIX(100),FW_FIX(100)},		// LayoutSize
	
	{						// Start list of frame's subviews
		FW_RStaticText
		(
			0,							// view id
			FW_BOUNDS(20, 40, 140, 60),
			FW_kFixedBounds,			// binding
			FW_NORMAL_FONT,
			FW_RGB_LIGHT_GRAY,			// erase color
			"Pick a password:"			
		),
		FW_RStaticText 
		(
			0,							// view id
			FW_BOUNDS(20, 70, 140, 90),
			FW_kFixedBounds,			// binding
			FW_NORMAL_FONT, 
			FW_RGB_LIGHT_GRAY,			// erase color
			"Retype to verify:"			
		),
		RPwdEditView		// edit view to enter new password
		(
			kPasswordEditID,			// view id
			FW_BOUNDS(150, 40, 280, 60),
			FW_kFixedBounds,			// binding
			10,							// max chars
			9,							// attributes
			FW_NORMAL_FONT, 
			""							// initial text
		), 
		RPwdEditView		// edit view to confirm password
		(
			kConfirmEditID,				// view id
			FW_BOUNDS(150, 70, 280, 90),
			FW_kFixedBounds,			// binding
			10,							// max chars
			9,							// attributes
			FW_NORMAL_FONT, 
			""							// initial text
		), 
		FW_RButton
		(
			kOKButtonID,				// view id
			FW_BOUNDS(180, 120, 260, 148),
			FW_kFixedBounds,			// binding
			FW_kDefaultButtonMsg,		// "OK" message
			0,							// receiver = dialogFrame by default
			0,							// control value
			FW_kDefaultPushButton,		// button kind (with outline)
			FW_NORMAL_FONT,
			"OK"			
		),
		FW_RButton
		(
			kCancelButtonID,			// view id
			FW_BOUNDS(60, 123, 120, 145),
			FW_kFixedBounds,			// binding
			FW_kCancelButtonMsg,		// "Cancel" message
			0,							// receiver = dialogFrame by default
			0,							// control value
			FW_kPushButton,				// button kind
			FW_NORMAL_FONT,
			"Cancel"			
		)
	},						// End list of frame's subviews
	{ }						// no scroller
}; 
