//========================================================================================
//
//	File:				Commands.h
//	Release Version:	$ ODF 1 $
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef COMMANDS_H
#define COMMANDS_H

// ----- Framework Includes -----

#ifndef FWCLPCMD_H
#include "FWClpCmd.h"
#endif

#ifndef FWDRCMD_H
#include "FWDrCmd.h"
#endif

#ifndef FWBNDSTR_H
#include "FWBndStr.h"
#endif

//========================================================================================
// Forward Declarations
//========================================================================================

class CHelloContent;

//========================================================================================
// CHelloEditCommand
//========================================================================================

class CHelloEditCommand : public FW_CClipboardCommand
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:

	FW_DECLARE_AUTO(CHelloEditCommand)
	
	CHelloEditCommand(Environment* ev,
					  ODCommandID id,
					  CHelloContent* itsContent,
					  FW_CFrame* frame);

	virtual ~CHelloEditCommand();

//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void		UndoIt(Environment *ev);
	virtual void		RedoIt(Environment *ev);
	virtual void		SaveUndoState(Environment *ev);

//----------------------------------------------------------------------------------------
//	New API
//
private:
	void 				RemoveSelection(Environment* ev);
	void 				RestoreSelection(Environment* ev);
	void 				SwapSelection(Environment* ev);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	CHelloContent*		fHelloContent;
	FW_CString255		fSavedTextData;		// Saved data for undo/redo
};



//========================================================================================
//	class CHelloDragCommand
//========================================================================================

class CHelloDragCommand : public FW_CDragCommand
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:

	FW_DECLARE_AUTO(CHelloDragCommand)
	
	CHelloDragCommand(Environment* ev,
					  CHelloContent* content,
					  FW_CFrame* frame);
	virtual ~CHelloDragCommand();

//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void		UndoIt(Environment *ev);
	virtual void		RedoIt(Environment *ev);
	virtual void		SaveUndoState(Environment *ev);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	CHelloContent*		fHelloContent;
	FW_CString255		fSavedTextData;		// Saved data for undo/redo
};

//========================================================================================
//	class CHelloDropCommand
//========================================================================================

class CHelloDropCommand : public FW_CDropCommand
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:

	FW_DECLARE_AUTO(CHelloDropCommand)
	
	CHelloDropCommand(Environment *ev,
					  CHelloContent* content,
					  FW_CFrame* frame,
					  ODDragItemIterator* dropInfo, 
					  ODFacet* odFacet,
					  const FW_CPoint& dropPoint);

	virtual ~CHelloDropCommand();

//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void		UndoIt(Environment *ev);
	virtual void		RedoIt(Environment *ev);
	virtual void		SaveUndoState(Environment *ev);

//----------------------------------------------------------------------------------------
//	New API
//
private:
	void SwapSelection(Environment* ev);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	CHelloContent*		fHelloContent;
	FW_CString255		fSavedTextData;		// Saved data for undo/redo
};


#endif
