//========================================================================================
//
//	File:				FWIntSpc.h
//	Release Version:	$ ODF 2 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWINTSPC_H
#define FWINTSPC_H

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWINTERE_H
#include "FWIntere.h"
#endif

#ifndef FWTCOLL_H
#include "FWTColl.h"
#endif

#ifndef FWORDCOL_H
#include "FWOrdCol.h"
#endif

#ifndef FWTDSTRY_H
#include "FWTDstry.h"
#endif

#include "FWTMap.tpp"

//========================================================================================
// Forward declarations
//========================================================================================

class FW_MReceiver;
class FW_MNotifier;
class FW_CNotification;

//========================================================================================
// Type declarations
//========================================================================================

//----------------------------------------------------------------------------------------
// Notifier Types
//----------------------------------------------------------------------------------------

typedef unsigned long FW_PrivInterestTag;

typedef FW_TOrderedCollection<FW_PrivInterestTag> FW_CPrivInterestTagCollection;

struct FW_SPrivTaggableInterest
{
	FW_MReceiver*				fReceiver;
	FW_Message					fMessage;
	FW_PrivInterestTag				fTags[2];
	FW_CPrivInterestTagCollection* 	fTagCollection;
};

typedef FW_TOrderedCollection<FW_SPrivTaggableInterest> FW_CPrivTaggableInterestList;

struct FW_SPrivNotifierEntry
{
	FW_MNotifier* 					fNotifier;
	FW_CPrivTaggableInterestList* 	fTaggedInterestList;
};

typedef FW_TMap<FW_ObjectID, FW_SPrivNotifierEntry> FW_CPrivNotifierInterestMap;
typedef FW_TPair<FW_ObjectID, FW_SPrivNotifierEntry> FW_CPrivNotifierInterestPair;

//----------------------------------------------------------------------------------------
// Receiver Types
//----------------------------------------------------------------------------------------


struct FW_SPrivReceiverEntry
{
	FW_MReceiver*				fReceiver;
	FW_CInterestCollection* 	fInterestList;
};

typedef FW_TMap<FW_ObjectID, FW_SPrivReceiverEntry> FW_CPrivReceiverInterestMap;
typedef FW_TPair<FW_ObjectID, FW_SPrivReceiverEntry> FW_CPrivReceiverInterestPair;

//========================================================================================
// CLASS FW_CPrivInterestSpace
//========================================================================================

class FW_CPrivInterestSpace
{
private:
	FW_CPrivInterestSpace(const FW_CPrivInterestSpace& other);
	FW_CPrivInterestSpace& operator=(const FW_CPrivInterestSpace& other);
	
	friend class FW_MNotifier;
	friend class FW_MReceiver;
	friend class FW_TPrivDestroyer<FW_CPrivInterestSpace>;

private:
	FW_CPrivInterestSpace();
	~FW_CPrivInterestSpace();

protected:
	static FW_CPrivInterestSpace* Instance();
	
	FW_ObjectID		GetNextNotifierID();
	FW_ObjectID		GetNextReceiverID();
	
	FW_PrivInterestTag	GetNextTag();
	
	void	AddNotifierInterest(FW_MReceiver* receiver, const FW_CInterest& interest);
	void	AddReceiverInterest(FW_MReceiver* receiver, const FW_CInterest& interest);
	
	void	RemoveNotifierInterest(FW_MReceiver* receiver, const FW_CInterest& interest);
	void	RemoveReceiverInterest(FW_MReceiver* receiver, 
									const FW_CInterest& interest,
									FW_Boolean removeNotifierInterest);
										
	FW_Boolean			HasInterest(const FW_MReceiver* receiver, const FW_CInterest& interest);
	void 				Notify(Environment* ev, const FW_CNotification& notification);
	
	void				AddTagToInterest(FW_SPrivTaggableInterest* interest, FW_PrivInterestTag tag);
	FW_Boolean			RemoveTagFromInterest(FW_SPrivTaggableInterest* interest, FW_PrivInterestTag tag);
	
	FW_SPrivTaggableInterest*	TagInterests(FW_CPrivTaggableInterestList* interestList,
										FW_Message message,
										FW_PrivInterestTag tag,
										unsigned short& interestsTagged);
										
	void	SendNotificationToReceiver(Environment* ev, FW_MReceiver* receiver, const FW_CNotification& notification);			

	void	RemoveNotifier(FW_MNotifier* notifier);
	void	RemoveReceiver(FW_MReceiver* receiver);
	
	void	SetReceiverIsConnected(FW_MReceiver* receiver, FW_Boolean isConnected); 
				
private:
	static FW_CPrivInterestSpace*				fgInstance;
	static FW_TPrivDestroyer<FW_CPrivInterestSpace> fgDestroyer;
	
	FW_ObjectID								fLastNotifierID;
	FW_ObjectID								fLastReceiverID;
	FW_PrivInterestTag							fLastTag;
	FW_CPrivNotifierInterestMap					fNotifierInterestMap;
	FW_CPrivReceiverInterestMap					fReceiverInterestMap;
};

//----------------------------------------------------------------------------------------
//	FW_CPrivInterestSpace::GetNextNotifierID
//----------------------------------------------------------------------------------------
inline FW_ObjectID FW_CPrivInterestSpace::GetNextNotifierID()
{
	return ++fLastNotifierID;
}

//----------------------------------------------------------------------------------------
//	FW_CPrivInterestSpace::GetNextReceiverID
//----------------------------------------------------------------------------------------
inline FW_ObjectID FW_CPrivInterestSpace::GetNextReceiverID()
{
	return ++fLastReceiverID;
}

//----------------------------------------------------------------------------------------
//	FW_CPrivInterestSpace::GetNextTag
//----------------------------------------------------------------------------------------
inline FW_PrivInterestTag FW_CPrivInterestSpace::GetNextTag()
{
	return ++fLastTag;
}
#endif
