//========================================================================================
//
//	File:				FWBufSin.h
//	Release Version:	$ ODF 2 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWBUFSIN_H
#define FWBUFSIN_H

#include "SLBufSin.xh"

#ifndef FWRANSIN_H
#include "FWRanSin.h"
#endif

//========================================================================================
//	CLASS FW_PBufferedSink
//
//		Inherited API for operator->()
//			long GetReadableBytes();
//			void Read(in void * destination, in long count);
//			long GetWritableBytes();
//			void Write(in void* source, in long count);
//
//			long GetLength();
//			void SetLength(in long length);
//			long GetPosition();
//			void SetPosition(in long position);
//
//		Inherited API for operator->()
//			void Flush();
//
//========================================================================================

class FW_PBufferedSink : public FW_PRandomAccessSink
{
public:
	FW_PBufferedSink(Environment* ev, FW_ORandomAccessSink* theSink, long capacity = 1024);
		// Construct a buffered sink.  The buffered sink places a read/write buffer
		// between clients and the sink itself.  The capacity is the number of bytes 
		// in the buffer.

	virtual ~FW_PBufferedSink();

public:
	// Covariant overrides
	FW_OBufferedSink* 	GetRep()			const;
	FW_OBufferedSink* 	operator->()		const;
	operator 			FW_OBufferedSink*()	const;

private:
	FW_PBufferedSink(const FW_PBufferedSink&);
	FW_PBufferedSink& operator=(const FW_PBufferedSink&);
};

//----------------------------------------------------------------------------------------
//	FW_PBufferedSink::GetRep
//----------------------------------------------------------------------------------------
inline FW_OBufferedSink* FW_PBufferedSink::GetRep() const
{
	return (FW_OBufferedSink*)FW_PRandomAccessSink::GetRep();
}

//----------------------------------------------------------------------------------------
//	FW_PBufferedSink::operator->
//----------------------------------------------------------------------------------------
inline FW_OBufferedSink* FW_PBufferedSink::operator->() const
{
	return GetRep();
}

//----------------------------------------------------------------------------------------
//	FW_PBufferedSink::operator FW_OBufferedSink*
//----------------------------------------------------------------------------------------
inline FW_PBufferedSink::operator FW_OBufferedSink*() const
{
	return GetRep();
}

#endif
