//========================================================================================
//
//	File:				FWEvent.h
//	Release Version:	$ ODF 2 $
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWEVENT_H
#define FWEVENT_H

#ifndef FWODTYPS_H
#include "FWODTyps.h"
#endif

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

#if defined(FW_BUILD_WIN)  && !defined(_INC_WINDOWS)
#include <windows.h>
#endif	

//========================================================================================
// Forward class declarations
//========================================================================================

class FW_CPoint;
class ODFrame;
class ODFacet;
class FW_CEmbeddedMouseEvent;
class FW_CMapping;

//========================================================================================
//  Type definitions
//========================================================================================

#ifdef FW_BUILD_MAC
typedef long FW_PlatformEventTime;
#endif

#ifdef FW_BUILD_WIN
typedef DWORD FW_PlatformEventTime;
#endif

//========================================================================================
// CLASS FW_CSystemEvent
//========================================================================================
class FW_CSystemEvent
{
public:
	static const short kEmpty;
	static const short kExtend;		// Mac shift key, Windows shift key
	static const short kItem;		// Mac command key, Windows control key		[HLX] ???
	static const short kCopy;		// Mac option key, Windows control key		[HLX] ???
	static const short kControl;	// Mac control key, Windows control key

public:

	virtual ~FW_CSystemEvent();
	
	ODEventData* 					GetPlatformEvent();
	unsigned long 					GetMessage() const;
	FW_PlatformEventTime 			GetTime() const;
	ODPlatformWindow 				GetPlatformWindow() const;

	// ----- Modifiers
	unsigned long 					GetPlatformModifiers(Environment* ev) const;
	unsigned short					GetModifiers(Environment* ev) const;

	FW_Boolean						IsExtendModifier(Environment* ev) const;			// Mac shift key, Windows shift key
	FW_Boolean						IsItemModifier(Environment* ev) const;				// Mac command key, Windows control key
	FW_Boolean						IsCopyModifier(Environment* ev) const;				// Mac option key, Windows control key		[HLX] ???
	FW_Boolean						IsControlKeyModifier(Environment* ev) const;		// Mac control key, Windows control key

	void							ClearExtendModifier(Environment* ev);
	void							ClearItemModifier(Environment* ev);
	void							ClearCopyModifier(Environment* ev);
	void							ClearControlKeyModifier(Environment* ev);

	FW_Boolean						IsPropagated(Environment* ev) const;
	
protected:
	FW_CSystemEvent(Environment* ev, ODPlatformWindow theWindow, FW_PlatformEventTime time);
	FW_CSystemEvent(Environment* ev, ODEventData* theEvent, FW_Boolean propagated);
	
	FW_CSystemEvent(Environment* ev, const FW_CSystemEvent& other);
	
	ODEventData 		fEvent;
	ODPlatformWindow	fPlatformWindow;
	FW_Boolean			fPropagated;
};

//========================================================================================
// CLASS FW_CNullEvent
//========================================================================================

class FW_CNullEvent : public FW_CSystemEvent
{
public:
	
	FW_CNullEvent(Environment* ev, ODEventData* theEvent, FW_Boolean propagated);
	FW_CNullEvent(Environment* ev, ODPlatformWindow theWindow, FW_PlatformEventTime time);

	FW_CNullEvent(Environment* ev, const FW_CNullEvent& other);
	virtual		~FW_CNullEvent();
};

#ifdef FW_BUILD_MAC
//========================================================================================
// CLASS FW_CMacWindowEvent
//========================================================================================

class FW_CMacWindowEvent : public FW_CSystemEvent
{
public:
	
	FW_CMacWindowEvent(Environment* ev, ODEventData* theEvent, FW_Boolean propagated);
	virtual		~FW_CMacWindowEvent();
};
#endif

//========================================================================================
// CLASS FW_CMouseEvent
//========================================================================================

class FW_CMouseEvent : public FW_CSystemEvent
{
public:
	
	enum EButtonState
	{
		kMouseButtonDown, kMouseButtonUp
	};

	enum EWhichButton
	{
		kLeftMouseButton, kMiddleMouseButton, kRightMouseButton
	};

	enum ECoordinates
	{
		kScreen, kWindow, kFrame
	};

	FW_CMouseEvent(Environment* ev, 
						ODEventData* theEvent, 
						ODFacet* theFacet, 
						EButtonState theState, 
						FW_Boolean propagated, 
						short numOfClicks = 1);
	FW_CMouseEvent(Environment* ev, 
						ODFacet* theFacet,
						EButtonState theState,
						EWhichButton initialButton,
						ODPlatformWindow theWindow,
						FW_PlatformEventTime when,
						const FW_CPoint& where,
						unsigned short modifiers,
						short numOfClicks = 1);
	FW_CMouseEvent(Environment* ev, const FW_CEmbeddedMouseEvent& embeddedEvent);
	
	FW_CMouseEvent(Environment* ev, const FW_CMouseEvent& other);
	virtual		~FW_CMouseEvent();
	
	
	ODFacet*			GetFacet(Environment* ev) const;
	EButtonState		GetButtonState(Environment* ev) const;
	EWhichButton		GetInitialButtonPressed(Environment* ev) const;
	short				GetNumberOfClicks(Environment* ev) const;

	// Position
	FW_CPoint			GetLogicalMousePosition(Environment* ev, 
										const FW_CMapping& mapping) const;
	FW_CPoint			GetMousePosition(Environment* ev, 
										ECoordinates coordinate) const;

	// Buttons Pressed
	FW_Boolean			IsLeftButtonPressed(Environment* ev) const;
	FW_Boolean			IsMiddleButtonPressed(Environment* ev) const;
	FW_Boolean			IsRightButtonPressed(Environment* ev) const;
	
	// Utility
	FW_Boolean			WaitUntilMouseMoved(Environment* ev) const;
	
protected:
	ODFacet* 			fFacet;
	EButtonState 		fButtonState;
	EWhichButton 		fWhichButton;
	short				fNumberOfClicks;
};

//========================================================================================
// CLASS FW_CEmbeddedMouseEvent
//========================================================================================

class FW_CEmbeddedMouseEvent : public FW_CMouseEvent
{
public:
	
	FW_CEmbeddedMouseEvent(Environment* ev, ODEventData* theEvent,
											ODFacet* theFacet,
											ODFrame* theEmbeddedFrame,
											ODFacet* theEmbeddedFacet,
											EButtonState theState,
											FW_Boolean propagated,
											short numOfClicks = 1);
											
	FW_CEmbeddedMouseEvent(Environment* ev, ODFacet* theFacet,
											ODFrame* theEmbeddedFrame,
											ODFacet* theEmbeddedFacet,
											EButtonState theState,
											EWhichButton initialButton,
											ODPlatformWindow theWindow,
											FW_PlatformEventTime when,
											const FW_CPoint& where,
											unsigned short modifiers,
											short numOfClicks = 1);

	FW_CEmbeddedMouseEvent(Environment* ev, const FW_CEmbeddedMouseEvent& other);
	virtual		~FW_CEmbeddedMouseEvent();
	
	ODFrame* 	GetEmbeddedFrame(Environment* ev) const;
	ODFacet* 	GetEmbeddedFacet(Environment* ev) const;

private:
	ODFrame* 	fEmbeddedFrame;
	ODFacet* 	fEmbeddedFacet;
};

//========================================================================================
// CLASS FW_CBorderMouseEvent
//========================================================================================

class FW_CBorderMouseEvent : public FW_CMouseEvent
{
public:
	
	FW_CBorderMouseEvent(Environment* ev, ODEventData* theEvent,
											ODFacet* theFacet,
											ODFrame* theEmbeddedFrame,
											ODFacet* theEmbeddedFacet,
											EButtonState theState,
											FW_Boolean propagated,
											short numOfClicks = 1);
											
	FW_CBorderMouseEvent(Environment* ev, ODFacet* theFacet,
											ODFrame* theEmbeddedFrame,
											ODFacet* theEmbeddedFacet,
											EButtonState theState,
											EWhichButton initialButton,
											ODPlatformWindow theWindow,
											FW_PlatformEventTime when,
											const FW_CPoint& where,
											unsigned short modifiers,
											short numOfClicks = 1);
											
	FW_CBorderMouseEvent(Environment* ev, const FW_CBorderMouseEvent& other);
	virtual		~FW_CBorderMouseEvent();

	ODFrame* 	GetEmbeddedFrame(Environment* ev) const;
	ODFacet* 	GetEmbeddedFacet(Environment* ev) const;

private:
	ODFrame* 	fEmbeddedFrame;
	ODFacet* 	fEmbeddedFacet;
};

//========================================================================================
// CLASS FW_CVirtualKeyEvent
//========================================================================================

class FW_CVirtualKeyEvent : public FW_CSystemEvent
{
public:
	
	enum KeyboardState
	{
		kKeyDown, kKeyUp
	};
	
	FW_CVirtualKeyEvent(Environment* ev, 
						ODEventData* theEvent, 
						KeyboardState theKeyboardState, 
						FW_Boolean propagated,
						unsigned short repeatCount = 0);
	FW_CVirtualKeyEvent(Environment* ev, 
						KeyboardState theKeyboardState,
						short keyCode,
						ODPlatformWindow theWindow,
						FW_PlatformEventTime when,
						unsigned short repeatCount = 0);

	FW_CVirtualKeyEvent(Environment* ev, const FW_CVirtualKeyEvent& other);
	virtual~ FW_CVirtualKeyEvent();
	
	KeyboardState	GetKeyboardState(Environment* ev) const;
	FW_Boolean 		IsRepeating(Environment* ev) const;
	unsigned short 	GetRepeatCount(Environment* ev) const;
	short 			GetKeyCode(Environment* ev) const;
	FW_Boolean 		IsCommandPeriod(Environment* ev) const;
	
private:
	KeyboardState 	fKeyboardState;
	unsigned short 	fRepeatCount;
};


//========================================================================================
// CLASS FW_CCharKeyEvent
//========================================================================================

class FW_CCharKeyEvent : public FW_CSystemEvent
{
public:
	
	FW_CCharKeyEvent(Environment* ev, 
					ODEventData* theEvent, 
					FW_Boolean propagated,
					unsigned short repeatCount = 0);
	FW_CCharKeyEvent(Environment* ev, 
					char keyChar,
					ODPlatformWindow theWindow,
					FW_PlatformEventTime when,
					unsigned short repeatCount = 0);

	FW_CCharKeyEvent(Environment* ev, const FW_CCharKeyEvent& other);
	virtual~ FW_CCharKeyEvent();

	FW_Boolean 		IsRepeating(Environment* ev) const;
	unsigned short 	GetRepeatCount(Environment* ev) const;
	char			GetChar(Environment* ev) const;
	
private:
	unsigned short fRepeatCount;
};

//========================================================================================
// CLASS FW_CSuspendResumeEvent
//========================================================================================

class FW_CSuspendResumeEvent : public FW_CSystemEvent
{
public:
	
	FW_CSuspendResumeEvent(Environment* ev, 
							ODEventData* theEvent, 
							ODFacet* facet, 
							FW_Boolean isGoingToBackground,
							FW_Boolean propagated);
	FW_CSuspendResumeEvent(Environment* ev, 
							const FW_CSuspendResumeEvent& other);
	virtual~ FW_CSuspendResumeEvent();
	
	FW_Boolean 		IsGoingToBackground(Environment* ev) const;
	ODFacet* 		GetFacet(Environment* ev) const;
	
private:
	FW_Boolean 		fIsGoingToBackground;
	ODFacet*		fFacet;
};

//========================================================================================
// CLASS FW_CActivateEvent
//========================================================================================

class FW_CActivateEvent : public FW_CSystemEvent
{
public:
	
	FW_CActivateEvent(Environment* ev, 
					ODEventData* theEvent, 
					ODFacet* facet, 
					FW_Boolean theState,
					FW_Boolean propagated);
	FW_CActivateEvent(Environment* ev, 
					ODFacet* facet,
					FW_Boolean theState,
					ODPlatformWindow theWindow,
					FW_PlatformEventTime when);

	FW_CActivateEvent(Environment* ev, const FW_CActivateEvent& other);
	virtual~ FW_CActivateEvent();
	
	FW_Boolean 					IsActivating(Environment* ev) const;
	ODFacet* 					GetFacet(Environment* ev) const;
	
private:
	FW_Boolean 		fActivationState;
	ODFacet*		fFacet;
};

//========================================================================================
// CLASS FW_CMenuEvent
//========================================================================================

class FW_CMenuEvent : public FW_CSystemEvent
{
public:
	
	FW_CMenuEvent(Environment* ev, 
					ODEventData* theEvent, 
					ODCommandID theCommandID,
					FW_Boolean propagated);
	FW_CMenuEvent(Environment* ev, 
					ODCommandID theCommandID,
					ODPlatformWindow theWindow,
					FW_PlatformEventTime when);

	FW_CMenuEvent(Environment* ev, const FW_CMenuEvent& other);
	virtual~ FW_CMenuEvent();
	
	ODCommandID GetCommandID(Environment* ev) const;
	
private:
	ODCommandID fCommandID;
};

//========================================================================================
// inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CSystemEvent::GetPlatformEvent
//----------------------------------------------------------------------------------------
inline ODEventData* FW_CSystemEvent::GetPlatformEvent()
{
	return &fEvent;
}

//----------------------------------------------------------------------------------------
//	FW_CSystemEvent::GetMessage
//----------------------------------------------------------------------------------------
inline unsigned long FW_CSystemEvent::GetMessage() const
{
	return fEvent.message;
}

//----------------------------------------------------------------------------------------
//	FW_CSystemEvent::GetTime
//----------------------------------------------------------------------------------------
inline FW_PlatformEventTime FW_CSystemEvent::GetTime() const
{
#ifdef FW_BUILD_MAC
	return fEvent.when;
#endif
#ifdef FW_BUILD_WIN
	return fEvent.time;
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CSystemEvent::GetPlatformModifiers
//----------------------------------------------------------------------------------------
inline unsigned long FW_CSystemEvent::GetPlatformModifiers(Environment*) const
{
#ifdef FW_BUILD_MAC
	return fEvent.modifiers;
#endif
#ifdef FW_BUILD_WIN
	return fEvent.wParam;
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CSystemEvent::GetPlatformWindow
//----------------------------------------------------------------------------------------
inline ODPlatformWindow FW_CSystemEvent::GetPlatformWindow() const
{
	return fPlatformWindow;
}

//----------------------------------------------------------------------------------------
//	FW_CSystemEvent::IsCopyModifier
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CSystemEvent::IsCopyModifier(Environment*) const
{
#ifdef FW_BUILD_MAC
	return (fEvent.modifiers & optionKey) != 0;
#endif
#ifdef FW_BUILD_WIN
	return (fEvent.wParam & MK_CONTROL) != 0;
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CSystemEvent::IsControlKeyModifier
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CSystemEvent::IsControlKeyModifier(Environment*) const
{
#ifdef FW_BUILD_MAC
	return (fEvent.modifiers & controlKey) != 0;
#endif
#ifdef FW_BUILD_WIN
	return (fEvent.wParam & MK_CONTROL) != 0;
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CSystemEvent::IsExtendModifier
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CSystemEvent::IsExtendModifier(Environment*) const
{
#ifdef FW_BUILD_MAC
	return (fEvent.modifiers & shiftKey) != 0;
#endif
#ifdef FW_BUILD_WIN
	return (fEvent.wParam & MK_SHIFT) != 0;
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CSystemEvent::IsItemModifier
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CSystemEvent::IsItemModifier(Environment*) const
{
#ifdef FW_BUILD_MAC
	return (fEvent.modifiers & cmdKey) != 0;
#endif
#ifdef FW_BUILD_WIN
	return (fEvent.wParam & MK_CONTROL) != 0;
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CSystemEvent::ClearExtendModifier
//----------------------------------------------------------------------------------------
inline void FW_CSystemEvent::ClearExtendModifier(Environment*)
{
#ifdef FW_BUILD_MAC
	fEvent.modifiers -= shiftKey;
#endif
#ifdef FW_BUILD_WIN
	fEvent.wParam -= MK_SHIFT;
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CSystemEvent::ClearItemModifier
//----------------------------------------------------------------------------------------
inline void FW_CSystemEvent::ClearItemModifier(Environment*)
{
#ifdef FW_BUILD_MAC
	fEvent.modifiers -= cmdKey;
#endif
#ifdef FW_BUILD_WIN
	fEvent.wParam -= MK_CONTROL;
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CSystemEvent::ClearCopyModifier
//----------------------------------------------------------------------------------------
inline void FW_CSystemEvent::ClearCopyModifier(Environment*)
{
#ifdef FW_BUILD_MAC
	fEvent.modifiers -= optionKey;
#endif
#ifdef FW_BUILD_WIN
	fEvent.wParam -= MK_CONTROL;
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CSystemEvent::ClearControlKeyModifier
//----------------------------------------------------------------------------------------
inline void FW_CSystemEvent::ClearControlKeyModifier(Environment*)
{
#ifdef FW_BUILD_MAC
	fEvent.modifiers -= controlKey;
#endif
#ifdef FW_BUILD_WIN
	fEvent.wParam -= MK_CONTROL;
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CMouseEvent::GetFacet
//----------------------------------------------------------------------------------------
inline ODFacet* FW_CMouseEvent::GetFacet(Environment*) const
{
	return fFacet;
}

//----------------------------------------------------------------------------------------
//	FW_CMouseEvent::GetButtonState
//----------------------------------------------------------------------------------------
inline FW_CMouseEvent::EButtonState FW_CMouseEvent::GetButtonState(Environment*) const
{
	return fButtonState;
}

//----------------------------------------------------------------------------------------
//	FW_CMouseEvent::GetInitialButtonPressed
//----------------------------------------------------------------------------------------
inline FW_CMouseEvent::EWhichButton FW_CMouseEvent::GetInitialButtonPressed(Environment*) const
{
	return fWhichButton;
}

//----------------------------------------------------------------------------------------
//	FW_CMouseEvent::GetNumberOfClicks
//----------------------------------------------------------------------------------------
inline short FW_CMouseEvent::GetNumberOfClicks(Environment*) const
{
	return fNumberOfClicks;
}

//----------------------------------------------------------------------------------------
//	FW_CVirtualKeyEvent::GetKeyboardState
//----------------------------------------------------------------------------------------
inline FW_CVirtualKeyEvent::KeyboardState FW_CVirtualKeyEvent::GetKeyboardState(Environment*) const
{
	return fKeyboardState;
}

//----------------------------------------------------------------------------------------
//	FW_CVirtualKeyEvent::IsRepeating
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CVirtualKeyEvent::IsRepeating(Environment*) const
{
	return (fRepeatCount > 0);
}

//----------------------------------------------------------------------------------------
//	FW_CVirtualKeyEvent::GetRepeatCount
//----------------------------------------------------------------------------------------
inline unsigned short FW_CVirtualKeyEvent::GetRepeatCount(Environment*) const
{
	return fRepeatCount;
}

//----------------------------------------------------------------------------------------
//	FW_CCharKeyEvent::IsRepeating
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CCharKeyEvent::IsRepeating(Environment*) const
{
	return (fRepeatCount > 0);
}

//----------------------------------------------------------------------------------------
//	FW_CCharKeyEvent::GetRepeatCount
//----------------------------------------------------------------------------------------
inline unsigned short FW_CCharKeyEvent::GetRepeatCount(Environment*) const
{
	return fRepeatCount;
}

//----------------------------------------------------------------------------------------
//	FW_CActivateEvent::IsActivating
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CActivateEvent::IsActivating(Environment*) const
{
	return fActivationState;
}

//----------------------------------------------------------------------------------------
//	FW_CActivateEvent::GetFacet
//----------------------------------------------------------------------------------------
inline ODFacet* FW_CActivateEvent::GetFacet(Environment*) const
{
	return fFacet;
}

//----------------------------------------------------------------------------------------
//	FW_CSuspendResumeEvent::IsGoingToBackground
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CSuspendResumeEvent::IsGoingToBackground(Environment*) const
{
	return fIsGoingToBackground;
}

//----------------------------------------------------------------------------------------
//	FW_CSuspendResumeEvent::GetFacet
//----------------------------------------------------------------------------------------
inline ODFacet* FW_CSuspendResumeEvent::GetFacet(Environment*) const
{
	return fFacet;
}

//----------------------------------------------------------------------------------------
//	FW_CMenuEvent::GetCommandID
//----------------------------------------------------------------------------------------
inline ODCommandID FW_CMenuEvent::GetCommandID(Environment*) const
{
	return fCommandID;
}

//----------------------------------------------------------------------------------------
//	FW_CSystemEvent::IsPropagated
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CSystemEvent::IsPropagated(Environment*) const
{
	return fPropagated;
}

#endif


