//========================================================================================
//
//	File:				FWEventH.h
//	Release Version:	$ ODF 2 $
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWEVENTH_H
#define FWEVENTH_H

#ifndef FWODTYPS_H
#include "FWODTyps.h"
#endif

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

#ifndef FWTCOLL_H
#include "FWTColl.h"
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CNullEvent;
class FW_CMouseEvent;
class FW_CEmbeddedMouseEvent;
class FW_CBorderMouseEvent;
class FW_CVirtualKeyEvent;
class FW_CCharKeyEvent;
class FW_CActivateEvent;
class FW_CSuspendResumeEvent;
class FW_CMenuEvent;
class FW_CMacWindowEvent;
class FW_CMenuBar;
class FW_MEventHandler;
class FW_CFrame;

//========================================================================================
//	 typedef
//========================================================================================

typedef unsigned long FW_Priority;

const FW_Priority FW_kNoPriority = 1000;		

//========================================================================================
//	 class FW_MEventHandler
//========================================================================================

class FW_MEventHandler
{
public:
	FW_DECLARE_CLASS

	typedef FW_MEventHandler* FW_MEventHandlerPtr;
	typedef ODID EventHandlerID;

//----------------------------------------------------------------------------------------
//	Constructors/Destructor
//
protected:
	FW_MEventHandler();
	FW_MEventHandler(Environment* ev, FW_MEventHandler* nextEventHandler, FW_Priority thePriority);

public:
	virtual ~ FW_MEventHandler();

//----------------------------------------------------------------------------------------
//	New API
//
public:
	FW_MEventHandler*			GetNextEventHandler(Environment* ev) const;
	void						SetNextEventHandler(Environment* ev, FW_MEventHandler* nextEventHandler);
		
	virtual FW_Priority			GetPriority(Environment* ev) const;
	virtual void				SetPriority(Environment* ev, FW_Priority thePriority);
	
	FW_Boolean					IsEnabled(Environment* ev) const;
	virtual void				Enable(Environment* ev);
	virtual void				Disable(Environment* ev);

	// ----- Frame Access -----

	virtual FW_CFrame*			GetFrame(Environment* ev) const;
	
	// ----- Event handler management -----

	virtual void 				AdoptEventHandler(Environment* ev, FW_MEventHandler* eventHandler);
	virtual FW_MEventHandler* 	RemoveEventHandler(Environment* ev, FW_MEventHandler* eventHandler);

	// ----- Target Managment -----
	
	FW_Boolean					IsTarget(Environment* ev) const;
	FW_Boolean					IsActiveTarget(Environment* ev) const;
	FW_Boolean					IsSuspendedTarget(Environment* ev) const;
	
	FW_Boolean					BecomeTarget(Environment* ev, FW_Boolean tabSelection = false);
	FW_Boolean					ResignTarget(Environment* ev);
	virtual FW_Boolean			WantsToBeTarget(Environment* ev);
	virtual long				WillingToResignTarget(Environment* ev);
	
	virtual void				ActivateTarget(Environment* ev, FW_Boolean tabSelection);
	virtual void				DeactivateTarget(Environment* ev);

	// ----- Event Handling Protocal -----

	virtual FW_Handled 			HandleIdle(Environment* ev, const FW_CNullEvent& theNullEvent);
	virtual FW_Handled 			DoIdle(Environment* ev, const FW_CNullEvent& theNullEvent);
	
	virtual FW_Handled 			HandleMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	virtual FW_Handled			DoMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);

	virtual FW_Handled 			HandleBGMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	virtual FW_Handled 			DoBGMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	
	virtual FW_Handled 			HandleMouseUp(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	virtual FW_Handled 			DoMouseUp(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	
	virtual FW_Handled 			HandleMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	virtual FW_Handled 			DoMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);

	virtual FW_Handled 			HandleBGMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	virtual FW_Handled 			DoBGMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);

	virtual FW_Handled 			HandleMouseUpInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	virtual FW_Handled 			DoMouseUpInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	
	virtual FW_Handled 			HandleMouseDownInEmbeddedFrameBorder(Environment* ev, const FW_CBorderMouseEvent& theBorderMouseEvent);
	virtual FW_Handled 			DoMouseDownInEmbeddedFrameBorder(Environment* ev, const FW_CBorderMouseEvent& theBorderMouseEvent);

	virtual FW_Handled 			HandleVirtualKey(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent);
	virtual FW_Handled 			DoVirtualKey(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent);

	virtual FW_Handled 			HandleCharKey(Environment* ev, const FW_CCharKeyEvent& theCharKeyEvent);
	virtual FW_Handled 			DoCharKey(Environment* ev, const FW_CCharKeyEvent& theCharKeyEvent);

	virtual FW_Handled 			HandleMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent);
	virtual FW_Handled 			DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent);

	virtual FW_Handled 			HandleAdjustMenus(Environment *ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot);
	virtual FW_Handled 			DoAdjustMenus(Environment *ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot);

//------------------------------------------------------------------------------------
//	Internal use Only
//
public:
	// ----- Dispatching -----
	FW_Handled				PrivDispatchMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	FW_Handled				PrivDispatchBGMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	FW_Handled				PrivDispatchMouseUp(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	FW_Handled 				PrivDispatchIdle(Environment* ev, const FW_CNullEvent& theNullEvent);
	FW_Handled 				PrivDispatchMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	FW_Handled 				PrivDispatchBGMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	FW_Handled 				PrivDispatchMouseUpInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	FW_Handled 				PrivDispatchMouseDownInEmbeddedFrameBorder(Environment* ev, const FW_CBorderMouseEvent& theBorderMouseEvent);
	FW_Handled 				PrivDispatchVirtualKey(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent);
	FW_Handled 				PrivDispatchCharKey(Environment* ev, const FW_CCharKeyEvent& theCharKeyEvent);
	FW_Handled 				PrivDispatchMenuEvent(Environment* ev, const FW_CMenuEvent& theMenuEvent);
	FW_Handled 				PrivDispatchAdjustMenus(Environment *ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot);
	
private:
	void					PrivInsert(Environment* ev, FW_MEventHandler* eventHandler);
		
//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	FW_MEventHandler*							fNextEventHandler;
	FW_Boolean									fEnabled;
	FW_Priority									fPriority;
	FW_TOrderedCollection<FW_MEventHandler>* 	fEventHandlers;
};

//========================================================================================
//	 Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	 FW_MEventHandler::GetNextEventHandler()
//----------------------------------------------------------------------------------------
inline FW_MEventHandler* FW_MEventHandler::GetNextEventHandler(Environment*) const
{
	return fNextEventHandler;
}

//----------------------------------------------------------------------------------------
//	 FW_MEventHandler::SetNextEventHandler()
//----------------------------------------------------------------------------------------
inline void FW_MEventHandler::SetNextEventHandler(Environment*, FW_MEventHandler* nextEventHandler)
{
	fNextEventHandler = nextEventHandler;
}

//----------------------------------------------------------------------------------------
//	 FW_MEventHandler::IsEnabled()
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_MEventHandler::IsEnabled(Environment*) const
{
	return fEnabled;
}

#endif
