//========================================================================================
//
//	File:				FWAcqScr.h
//	Release Version:	$ ODF 2 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWACQSCR_H
#define FWACQSCR_H

//========================================================================================
// Forward Class Declarations
//========================================================================================

class FW_MScriptable;

//========================================================================================
// class FW_TAcquiredScriptableObject
//========================================================================================

template<class t> 
class FW_TAcquiredScriptableObject
{
public:
	FW_DECLARE_AUTO(FW_TAcquiredScriptableObject<t>)

	FW_TAcquiredScriptableObject() :
		fScriptableObject(NULL) {}
		
	FW_TAcquiredScriptableObject(const FW_TAcquiredScriptableObject<t>& other) :
		fScriptableObject(other.fScriptableObject) 
			{ if (fScriptableObject != NULL) fScriptableObject->AcquireScriptable(); }
		
	FW_TAcquiredScriptableObject(t* other) :
		fScriptableObject(other) {}
		
	~FW_TAcquiredScriptableObject() 
		{ if (fScriptableObject != NULL) fScriptableObject->ReleaseScriptable(); }

	FW_TAcquiredScriptableObject<t>& operator=(const FW_TAcquiredScriptableObject<t>& other);
	FW_TAcquiredScriptableObject<t>& operator=(t* other);

	FW_Boolean operator==(const FW_TAcquiredScriptableObject<t>& other) const
		{return fScriptableObject == other.fScriptableObject;}
	FW_Boolean operator!=(const FW_TAcquiredScriptableObject<t>& other) const
		{return fScriptableObject != other.fScriptableObject;}

	FW_Boolean operator==(const t* other) const
		{return fScriptableObject == other;}
	FW_Boolean operator!=(const t* other) const
		{return fScriptableObject != other;}

	operator t*() const
		{return fScriptableObject;}		
	operator const void*() const
		{return fScriptableObject;}		
	t* operator->() const
		{return fScriptableObject;}
private:
	t* fScriptableObject;
};

template <class t>
FW_TAcquiredScriptableObject<t>& FW_TAcquiredScriptableObject<t>::operator=(const FW_TAcquiredScriptableObject<t>& other)
{
	if (fScriptableObject != other)
	{
		if (fScriptableObject != NULL)
			fScriptableObject->ReleaseScriptable();
		
		fScriptableObject = NULL;	// in case acquisition throws
		
		if (other != NULL)
		{
			other->AcquireScriptable();
			fScriptableObject = other;
		}
	}
	return *this;
}

template <class t>
FW_TAcquiredScriptableObject<t>& FW_TAcquiredScriptableObject<t>::operator=(t* other)
{
	if (fScriptableObject != other)
	{
		if (fScriptableObject != NULL)
		{
			t* temp = fScriptableObject;
			fScriptableObject = NULL;	// in case release throws
			temp->ReleaseScriptable();
		}
		
		fScriptableObject = other;
	}
	return *this;
}

//========================================================================================
// Template classes typedef
//========================================================================================

#ifndef FWSCPTBL_H
#include "FWScptbl.h"
#endif
typedef FW_TAcquiredScriptableObject<FW_MScriptable>	FW_CAcquiredScriptable;

#ifndef FWSCPCOL_H
#include "FWScpCol.h"
#endif
typedef FW_TAcquiredScriptableObject<FW_CScriptableCollection> FW_CAcquiredScriptableCollection;

#endif
