//========================================================================================
//
//	File:				Part.h
//	Release Version:	$ ODF 2 $ 
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef TBLPART_H
#define TBLPART_H

// ----- TablePart Includes -----

#ifndef DEFINES_K
#include "Defines.k"
#endif

// ----- ODF Includes -----

#ifndef FWPARTNG_H
#include "FWPartng.h"
#endif

#ifndef FWMNUITM_H
#include "FWMnuItm.h"
#endif

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

#ifndef FWFXMATH_H
#include "FWFxMath.h"
#endif

//========================================================================================
//	Forward declarations
//========================================================================================

class CTableFrame;
class CTableSelection;
class CTableLinkManager;
class CCell;
class CTablePartContent;

//========================================================================================
//	class CTablePart
//========================================================================================

class CTablePart : public FW_CEmbeddingPart
{
  public:
	friend class CTableLinkManager;	

	FW_DECLARE_AUTO(CTablePart)

//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	CTablePart(ODPart* odPart);
	virtual ~CTablePart();

//----------------------------------------------------------------------------------------
//	API Inherited from FW_CPart
//
public:
	virtual void				Initialize(Environment* ev, ODStorageUnit* storageUnit, FW_Boolean fromStorage);
				
	// ----- Factory Methods -----
	virtual FW_CFrame* 			NewFrame(Environment* ev, 
								 	ODFrame* odFrame, 
								 	FW_CPresentation* presentation,
								 	FW_Boolean fromStorage);
	virtual FW_CContent* 		NewPartContent(Environment* ev);
	virtual FW_CLinkManager*	NewLinkManager(Environment* ev);	// Override

										
	// ----- About -----
	virtual FW_Handled			DoAbout(Environment* ev);
	
//----------------------------------------------------------------------------------------
//	New API
//
public:
  	// ----- Other Initialization -----
	void 				DoPostCreate(Environment* ev);

	// ----- Change -----
	void				PartChanged(Environment* ev);
	
	CTablePartContent*		GetTableContent(Environment*) const
								{return fTableContent;}
	FW_CPresentation*	GetTablePresentation(Environment*) const
								{return fTablePresentation;}
								
	// ----- Linking Support -----
	CTableSelection*	GetTableSelection(Environment* ev);

	void				AboutToClearCell(Environment* ev, const CCell& cell);
	void				ChangeFrameStatus(Environment* ev, const CCell& cell, ODLinkStatus newStatus);
	void				ProxyMoved(Environment* ev, const CCell& fromCell, const CCell& toCell);

	// ----- Frames -----
	FW_Boolean			IsFrameValid(Environment* ev, FW_CFrame* frame);
	CTableFrame*		FindValidTableFrame(Environment* ev, CTableFrame* frame1, CTableFrame* frame2);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
  	CTablePartContent*		fTableContent;
 	FW_CPresentation*		fTablePresentation;
	
	CTableLinkManager*		fTableLinkManager;
	FW_Boolean				fPostCreated;
};

#endif
